/**
 * DefaultToolbar.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */

package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QToolBar;
import com.trolltech.qt.gui.QToolButton;

/**
 * The basic default toolbar for editing cards. It will include text-editing and
 * basic shapes and, eventually, multimedia.
 * 
 * @author Jesse Yates
 */
public class DefaultToolBar extends QToolBar {
	private List<QToolButton> buttons;
	public QToolButton undo;
	public QToolButton redo;
	public QToolButton changePenSize;
	public QToolButton changePenColor;

	/**
	 * default constructor
	 */
	public DefaultToolBar() {
		generateButtons();
		this.setFloatable(false);
		this.buttons = new ArrayList<QToolButton>();
	}

	/**
	 * Temporary method to create a set of buttons for the toolbar
	 */
	private void generateButtons() {
		QToolButton button = new QToolButton();
		button.setArrowType(Qt.ArrowType.LeftArrow);
		this.addWidget(button);
	}

	/**
	 * Set the action for the button
	 * 
	 * @param button
	 *            The button to set the action for
	 * @param action
	 *            the action the button to take
	 * @return <code> true </code> if it succeeds
	 */
	public boolean setAction(QToolButton button, QAction action) {
		if (!this.buttons.contains(button))
			return false;
		else {
			button = this.buttons.get(this.buttons.indexOf(button));
			button.addAction(action);
		}
		return true;
	}

	/**
	 * Get the list of the current buttons
	 * 
	 * @return An <code>unmodifiableList</code> (see
	 *         {@link java.util.Collections}) of the buttons
	 */
	public List<QToolButton> getButtons() {
		return Collections.unmodifiableList(this.buttons);
	}

}
