/**
 * DefaultCanvasScene.java
 * Group 7, Object Oriented Software Engineering
 * Nov 3, 2009
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QFile;
import com.trolltech.qt.core.QMimeData;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.QRectF;
import com.trolltech.qt.core.QUrl;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QClipboard;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFocusEvent;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsPixmapItem;
import com.trolltech.qt.gui.QGraphicsScene;
import com.trolltech.qt.gui.QGraphicsSceneDragDropEvent;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QGraphicsTextItem;
import com.trolltech.qt.gui.QImage;
import com.trolltech.qt.gui.QPen;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QTextCursor;
import com.trolltech.qt.gui.QUndoStack;
import com.trolltech.qt.gui.QGraphicsItem.GraphicsItemFlag;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.DrawCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.DrawItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.MoveItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop.DragDropItem;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop.DragDropMimeData;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop.DragDropPixmap;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop.FocusableTextItem;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems.DrawingItem;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems.DrawingSelectItem;

/**
 * @author Jesse Yates
 *         Underlying model for the the DrawingView. Deals with all the adding/removing of objects
 *         Currently VERY bad design - first concern is working implementation (conscious violation of OCP)
 */
public class DrawingGraphicsScene extends QGraphicsScene implements Serializable
{
	private static final long serialVersionUID = 8399101110101L;

	private boolean selectMode;

	/** amount to shift pasted items from original position */
	private static int pasteShift = 20;

	/** Whether or not it has been modified */
	private boolean isModified;

	/** Current width of the pen for shapes */
	private int myPenWidth;

	/** Current color of the pen for shapes */
	private transient QColor myPenColor;

	/** Whether or not text boxes added should be centered */
	private boolean centerText;

	/** Stack maintaining the undo/redo commands */
	private transient QUndoStack stack;

	/**
	 * Signals to indicate if there has been a mouse action
	 */
	private transient QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMousePress = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
	private transient QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMouseHold = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
	private transient QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMouseRelease = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();

	/** Indicates if the element has gained focus */
	public transient QSignalEmitter.Signal0 gainedFocus = new QSignalEmitter.Signal0();
	public transient QSignalEmitter.Signal0 lostFocus = new QSignalEmitter.Signal0();

	/** The current drawing item */
	private DrawingItem drawingItem;

	/**
	 * Default Constructor
	 */
	public DrawingGraphicsScene()
	{
		this.stack = new QUndoStack();
		this.centerText(false);
		this.myPenColor = new QColor(QColor.black);
		this.myPenWidth = 5;
		this.isModified = false;
		this.setFont(new QFont("Times New Roman", 12, 50));
		this.selectMode = false;
		this.changed.connect(this, "setModified(List)");
	}

	public DrawingGraphicsScene(List<QGraphicsItemInterface> items)
	{
		this();
		for(QGraphicsItemInterface i : items)
			super.addItem(i);
	}

	public DrawingGraphicsScene clone()
	{
		return new DrawingGraphicsScene(super.items());
	}

	public boolean equals(DrawingGraphicsScene other)
	{
		return super.items().equals(other.items());
	}

	public int hashCode()
	{
		return super.items().hashCode();
	}

	/**
	 * @return If the canvas has been modified
	 */
	public boolean isModified()
	{
		return isModified;
	}

	/**
	 * @param isModified
	 *        the isModified to set
	 */
	public void setModified(boolean isModified)
	{
		this.isModified = isModified;
	}

	/**
	 * @param isModified
	 *        the isModified to set
	 */
	public void setModified(List<QRectF> region)
	{
		this.setModified(true);
	}

	/**
	 * Whether or not to center the text in the panel
	 * 
	 * @param b
	 *        <code>true</code> if it should be centered, <code>false</code> otherwise
	 */
	public void centerText(boolean b)
	{
		this.centerText = b;
	}

	/**
	 * Get the pen color
	 * 
	 * @return The <code>QColor</code> of the pen
	 */
	public QColor penColor()
	{
		return myPenColor;
	}

	/**
	 * Get the pen width
	 * 
	 * @return the pen width
	 */
	public int penWidth()
	{
		return myPenWidth;
	}

	/**
	 * Set the color of the pen
	 * 
	 * @param newColor
	 *        The <code>QColor</code> of the pen
	 */
	public void setPenColor(QColor newColor)
	{
		this.myPenColor = newColor;
		this.updatePen();
	}

	/**
	 * Set the width of the pen
	 * 
	 * @param width
	 *        The new pen width
	 */
	public void setPenWidth(int width)
	{
		this.myPenWidth = width;
		this.updatePen();

	}

	private void updatePen()
	{
		if(this.drawingItem != null)
			this.drawingItem.setPen(new QPen(this.myPenColor, this.myPenWidth));
	}

	/**
	 * Handles mousePress events. Emits signal that is there is a left button click to any listening object/method
	 * If there are any <code>QGraphicsItemInterface</code> that are selected, they are unselected in the click, unless
	 * currently in select mode
	 */
	protected void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Mouse press");
		for(QGraphicsItemInterface i : this.items())
		{
			System.out.println("item: " + i);
			// i.mousePressEvent(event);
		}
		if(event.button() == Qt.MouseButton.LeftButton)
		{

			this.drawMousePress.emit(event);
			if(this.itemAt(event.scenePos()) != null)
			{

				System.out.println("item not null at location");
				System.out.println("SelectMode: " + this.selectMode);
				if(this.selectMode)
				{
					System.out.println("Propograting mouse click to non-null item");
					for(QGraphicsItemInterface i : this.selectedItems())
					{
						System.out.println("item: " + i);
						i.mousePressEvent(event);
					}
				}
				else
					super.mousePressEvent(event);

			}

			super.mousePressEvent(event);
		}
		update();
	}

	/**
	 * Handles mouseMove events. Emits signal that is there is a move to any listening object/method
	 */
	protected void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		this.drawMouseHold.emit(event);
		if(this.selectMode)
		{

			for(QGraphicsItemInterface i : this.selectedItems())
			{
				i.mouseMoveEvent(event);
			}

		}
		super.mouseMoveEvent(event);
	}

	/**
	 * Handles mouseRelease events. Emits signal that is there is a release to any listening object/method
	 */
	protected void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		this.drawMouseRelease.emit(event);
		if(this.selectMode)
		{
			for(QGraphicsItemInterface i : this.selectedItems())
			{
				System.out.println("selected: " + i);
				i.mouseReleaseEvent(event);
			}
		}
		super.mouseReleaseEvent(event);
		update();
		// DragDropUtilities.printScene("scene", this);
	}

	/**
	 * Slot for the lostFocus slot that <code>CardTextItem</code> has. Useful for removing a if there is no text in the
	 * box
	 * 
	 * @param item
	 */
	public void itemLostFocus(QGraphicsTextItem item)
	{
		QTextCursor cursor = item.textCursor();
		cursor.clearSelection();
		item.setTextCursor(cursor);

		if(item.toPlainText().length() == 0)
		{
			this.removeItem(item);
			// System.out.println("Removing item!");
		}
	}

	/**
	 * Set the current type based on the button that was pressed. Connects the proper slots to those objects
	 * 
	 * @param buttonPressed
	 *        the button pressed - needs to be a DrawingRadioButton
	 */
	public void setType(DrawingItem type)
	{
		this.drawMouseHold.disconnect(this.drawingItem);
		this.drawMousePress.disconnect(this.drawingItem);
		this.drawMouseRelease.disconnect(this.drawingItem);

		this.drawingItem = null;
		this.drawingItem = type.copy(this);
		this.drawingItem.setParent(this);

		if(this.drawingItem instanceof DrawingSelectItem)
			this.setSelectMode(true);
		else
			this.setSelectMode(false);

		this.drawMousePress.connect(this.drawingItem, "mousePressEvent(QGraphicsSceneMouseEvent)");
		this.drawMouseHold.connect(this.drawingItem, "mouseMoveEvent(QGraphicsSceneMouseEvent)");
		this.drawMouseRelease.connect(this.drawingItem, "mouseReleaseEvent(QGraphicsSceneMouseEvent)");
		this.drawingItem.eventItem.connect(this, "addEvent(QGraphicsItemInterface)");

		this.updatePen();

	}

	public void dropEvent(QGraphicsSceneDragDropEvent event)
	{
		importMimeData(event.mimeData(), event.scenePos());
		event.acceptProposedAction();
	}

	/**
	 * Undoes the last element on the stack
	 */
	public void undo()
	{
		if(this.hasFocus())
		{
			if(this.stack.canUndo())
				this.stack.undo();
			if(!this.stack.canUndo())
				this.setModified(false);
		}
	}

	/**
	 * Redoes the last element on the stack
	 */
	public void redo()
	{
		if(this.hasFocus())
			if(this.stack.canRedo())
				this.stack.redo();
	}

	/**
	 * /**
	 * Add an event to the stack
	 * 
	 * @param command
	 *        To be added. only accepted if it is a valid command
	 * @return <code>true</code> if the item is pushed on the stack, false otherwise
	 */
	public boolean addEvent(QGraphicsItemInterface item)
	{
		DrawCommand command = new DrawItemCommand(item, this);
		if(command.isValid())
		{
			this.stack.push(command);
			return true;
		}
		return false;
	}

	/**
	 * Add an event to the stack
	 * 
	 * @param command
	 *        To be added. only accepted if it is a valid command
	 * @return <code>true</code> if the item is pushed on the stack, false otherwise
	 */
	public boolean moveEvent(MoveItemCommand item)
	{
		// System.out.println("Adding item move");
		stack.push(item);
		return true;
	}

	/**
	 * Add an event to the stack
	 * 
	 * @param command
	 *        To be added. only accepted if it is a valid command
	 * @return <code>true</code> if the item is pushed on the stack, false otherwise
	 */
	public boolean resizeEvent(ResizeItemCommand item)
	{
		// System.out.println("Resize item");
		stack.push(item);
		return true;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		System.out.println("WRITING SCENE!");
		out.writeObject(super.items());
	}

	@SuppressWarnings("unchecked")
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException, ClassCastException
	{
		// Constructor
		this.stack = new QUndoStack();
		this.centerText(false);
		this.setPenColor(QColor.black);
		this.setPenWidth(5);
		this.setModified(false);
		this.setFont(new QFont("Times New Roman", 12, 50));
		this.selectMode = false;
		this.changed.connect(this, "setModified(List)");

		List<QGraphicsItemInterface> items = (List<QGraphicsItemInterface>) in.readObject();
		for(QGraphicsItemInterface item : items)
			super.addItem(item);

		this.gainedFocus = new QSignalEmitter.Signal0();
		this.lostFocus = new QSignalEmitter.Signal0();
		this.drawMousePress = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
		this.drawMouseHold = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
		this.drawMouseRelease = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
	}

	public String toString()
	{
		StringBuffer output = new StringBuffer();
		for(QGraphicsItemInterface i : this.items())
		{
			System.out.println("Item i:" + i);
		}

		return output.toString();
	}

	protected void focusInEvent(QFocusEvent event)
	{
		super.focusInEvent(event);
		this.gainedFocus.emit();
	}

	protected void focusOutEvent(QFocusEvent event)
	{
		super.focusOutEvent(event);
		this.lostFocus.emit();
	}

	/**
	 * @return the centerText
	 */
	public boolean isCenterText()
	{
		return centerText;
	}

	/**
	 * @return the selectMode
	 */
	public boolean isSelectMode()
	{
		return selectMode;
	}

	/**
	 * @param selectMode
	 *        the selectMode to set
	 */
	public void setSelectMode(boolean selectMode)
	{
		if(this.selectMode != selectMode)
		{
			this.selectMode = selectMode;
			System.out.println("SelectMode: " + this.selectMode);

			for(QGraphicsItemInterface item : this.items())
			{
				item.setFlag(GraphicsItemFlag.ItemIsSelectable, selectMode);// this.selectMode);
			}
			/*
			 * if(this.selectMode)
			 * System.out.println("Select mode on!");
			 * else
			 * System.out.println("Select mode off!");
			 */
		}
	}

	public void paste(QMimeData mimeData)
	{
		importMimeData(mimeData, new QPointF(0, 0));
	}

	private void importMimeData(QMimeData mimeData, QPointF pos)
	{
		System.out.println("Mime data: " + mimeData.text());

		System.out.println("color: " + mimeData.hasColor());
		if(mimeData.hasImage())
		{
			QImage image = new QImage();
			image = (QImage) mimeData.imageData();
			this.addPixmaptoScene(QPixmap.fromImage(image), pos);
		}
		else if(mimeData.hasText())
		{
			FocusableTextItem text = new FocusableTextItem(mimeData.text());
			text.setSelected(true);
			text.setPos(0, 0);
			this.addEvent(text);
		}
		else if(mimeData instanceof DragDropMimeData)
		{
			List<DragDropItem> items = ((DragDropMimeData) mimeData).getDragDropItem();
			for(DragDropItem i : items)
			{
				if(this.items().contains(i))
					i = i.copy();
				this.addEvent(i);
				i.setScene(this);
				i.setFlag(GraphicsItemFlag.ItemIsSelectable, true);
			}
		}
		else if(mimeData.hasUrls())
		{
			for(QUrl i : mimeData.urls())
			{
				if(!i.isEmpty())
				{
					if(QFile.exists(i.toLocalFile()))
					{
						QPixmap map = new QPixmap(i.toLocalFile());
						if(map != null)
							this.addPixmaptoScene(map, pos);
					}

				}

			}
		}
		return;
	}

	private void addPixmaptoScene(QPixmap map, QPointF pos)
	{
		QGraphicsPixmapItem pix = new QGraphicsPixmapItem();
		pix.setPixmap(map);
		pix.setPos(pos);
		pix.setFlag(GraphicsItemFlag.ItemIsSelectable, true);
		this.addEvent(new DragDropPixmap(pix, this));
		return;
	}

	/** Cut the currently selected item(s) onto the clipboard */
	public void cut(QClipboard clipboard)
	{
		System.out.println("Cut");
		if(!this.selectMode)
			return;
		DragDropMimeData data = new DragDropMimeData();

		for(QGraphicsItemInterface i : this.selectedItems())
		{
			if(i instanceof DragDropItem)
			{
				data.addDragDrop((DragDropItem) i);
				this.removeItem(i);
			}

		}
		clipboard.setMimeData(data);
	}

	/** Copy the currently selected item(s) onto the clipboard */
	public void copy(QClipboard clipboard)
	{
		System.out.println("Copy!");
		if(!this.selectMode)
			return;
		DragDropMimeData data = new DragDropMimeData();
		for(QGraphicsItemInterface i : this.selectedItems())
		{
			if(i instanceof DragDropItem)
			{
				data.addDragDrop(((DragDropItem) i).copy());
			}

		}
		clipboard.setMimeData(data);

	}
}
