/**
 * MyGraphicsView.java
 * Group 7, Object Oriented Software Engineering
 * Nov 3, 2009
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QDragEnterEvent;
import com.trolltech.qt.gui.QDragLeaveEvent;
import com.trolltech.qt.gui.QDragMoveEvent;
import com.trolltech.qt.gui.QDropEvent;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsTextItem;
import com.trolltech.qt.gui.QGraphicsView;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QPixmap;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems.DrawingTextItem;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;

/**
 * @author Jesse Yates
 *         Graphics view that should be able to handle input/output of image - doesnt yet
 */
public class DrawingGraphicsView extends QGraphicsView
{
	/**
	 * Constructor
	 * 
	 * @param model
	 *        the model for this view
	 */
	public DrawingGraphicsView(DrawingGraphicsScene model)
	{
		super(model);
		this.setSceneRect(0, 0, CardSection.WIDTH, CardSection.HEIGHT);
		this.setAcceptDrops(true);
	}

	/**
	 * Open an image from a file name - will populating the model from a string
	 * 
	 * @param fileName
	 *        The string to get the information from
	 */
	public void openImage(String fileName)
	{
		// TODO Auto-generated method stub

	}

	/**
	 * Get the current image display
	 * 
	 * @return <code>QPixmap</code> of the current view
	 */
	public QPixmap getImage()
	{
		QPixmap image = new QPixmap(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
		image.fill(QColor.white);

		QPainter painter = new QPainter();
		painter.begin(image);
		painter.setRenderHints(new QPainter.RenderHints(QPainter.RenderHint.Antialiasing, QPainter.RenderHint.SmoothPixmapTransform));
		this.scene().setSceneRect(0, 0, CardSection.WIDTH - 1, CardSection.HEIGHT - 1);
		this.scene().render(painter);
		painter.setPen(new QColor(160, 160, 160));
		painter.drawRect(0, 0, CardSection.WIDTH - 1, CardSection.HEIGHT - 1);
		painter.end();

		return image;
	}

	public QPixmap getScaledImage(int width, int height)
	{
		QPixmap map = this.getImage();
		return map.scaled(width, height, Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.FastTransformation);
	}

	/*
	 * public void drawImage(QWidget widget)
	 * {
	 * widget.resize(this.rect().size());
	 * QPainter painter = new QPainter();
	 * painter.begin(widget);
	 * //painter.setRenderHint(QPainter.RenderHint.Antialiasing);
	 * // print, fitting the viewport contents into a full page
	 * this.scene().render(painter);
	 * painter.end();
	 * }
	 * public QWidget drawImage()
	 * {
	 * return this.viewport();
	 * }
	 */

	protected void mouseMoveEvent(QMouseEvent event)
	{
		this.setToolTip(event.pos().toString() + "\n" + this.mapToScene(event.pos()).toString());
		super.mouseMoveEvent(event);
	}

	protected void dragEnterEvent(QDragEnterEvent event)
	{
		event.acceptProposedAction();
	}

	protected void dragMoveEvent(QDragMoveEvent event)
	{
		event.acceptProposedAction();
	}

	protected void dropEvent(QDropEvent event)
	{
		super.dropEvent(event);
		/*
		 * else if(mimeData.hasHtml())
		 * {
		 * textI.setHtml(mimeData.html());
		 * this.scene().addItem(textI);
		 * }
		 * else if(mimeData.hasText())
		 * {
		 * textI.setPlainText(mimeData.text());
		 * this.scene().addItem(textI);
		 * }
		 * else if(mimeData.hasUrls())
		 * {
		 * List<QUrl> urlList = mimeData.urls();
		 * String text = "";
		 * for(int i = 0; i < urlList.size() && i < 32; ++i)
		 * {
		 * String url = urlList.get(i).path();
		 * text += url + "\n";
		 * }
		 * textI.setPlainText(text);//setText(text);
		 * this.scene().addItem(textI);
		 * }
		 * else
		 * {
		 * textI.setPlainText(tr("Cannot display data"));
		 * this.scene().addItem(textI);
		 * }
		 */
		event.acceptProposedAction();
	}

	protected void dragLeaveEvent(QDragLeaveEvent event)
	{
		event.accept();
	}

	/**
	 * Get the concatenation of the all the strings in the scene
	 * 
	 * @return
	 */
	public String getStrings()
	{
		StringBuffer ret = new StringBuffer();
		for(QGraphicsItemInterface i : this.scene().items())
		{
			if(i instanceof DrawingTextItem || i instanceof QGraphicsTextItem)
			{
				ret.append(((QGraphicsTextItem) i).toPlainText());
				ret.append("\n");
			}
		}
		ret.trimToSize();
		return ret.toString();
	}
}
