/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.buttons;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.gui.QAbstractButton;
import com.trolltech.qt.gui.QButtonGroup;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems.DrawingItem;

/**
 * ButtonGroup that can emit the DrawingType that was changed to
 * 
 * @author Jesse Yates
 */
public class DrawingButtonGroup extends QButtonGroup
{
	/**
	 * Signal that indicates which drawingType has been changed to
	 */
	public final QSignalEmitter.Signal1<DrawingItem> typeChange = new QSignalEmitter.Signal1<DrawingItem>();

	/**
	 * Default constructor
	 */
	public DrawingButtonGroup()
	{
		this.buttonClicked.connect(this, "changeType(QAbstractButton)");
	}

	/**
	 * Emits the type that has been changed to if the button is a DrawingButton
	 * 
	 * @param b
	 */
	public void changeType(QAbstractButton b)
	{
		if(b instanceof DrawingButton)
		{
			DrawingButton a = (DrawingButton) b;
			typeChange.emit(a.getType());
		}
	}

	/**
	 * Gets the type that has been currently set in the group
	 * 
	 * @return <code>DrawingType</code> that has been currently set
	 */
	public DrawingItem getSetType()
	{
		for(QAbstractButton b : this.buttons())
		{
			if(b.isChecked())
			{
				if(b instanceof DrawingButton)
				{
					DrawingButton a = (DrawingButton) b;
					return a.getType();
				}
			}
		}
		return null;
	}
}
