/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.commands;

import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QUndoCommand;

/**
 * Movements commands that be pushed/popped from the stack
 * 
 * @author Jesse Yates
 */
public class MoveItemCommand extends QUndoCommand
{
	/** The old item position */
	QPointF oldPos;
	/** The new item position */
	QPointF newPos;
	/** The item that is being moved */
	QGraphicsItemInterface item;

	/**
	 * Constructor
	 * 
	 * @param oldPos
	 *        The previous item position
	 * @param item
	 *        The current item
	 */
	public MoveItemCommand(QPointF oldPos, QGraphicsItemInterface item)
	{
		this.oldPos = oldPos;
		this.newPos = item.pos();
		this.item = item;
	}

	/**
	 * Move the item back to the previous location
	 */
	public void undo()
	{
		this.item.setPos(oldPos);
		item.scene().update();
		//System.out.println("Old pos: " + this.item.pos());
	}

	/**
	 * Move the item back to the original location
	 */
	public void redo()
	{
		this.item.setPos(newPos);
		//item.scene().update();
		//System.out.println("New pos: " + this.item.pos());
	}

	/**
	 * Combine two moves if they are the same item over a set of points, with preference to the older location
	 */
	public boolean mergeWith(QUndoCommand command)
	{
		System.out.println("Merging move");
		MoveItemCommand moveCommand;
		if(command instanceof MoveItemCommand)
			moveCommand = (MoveItemCommand) command;
		else
			return false;
		QGraphicsItemInterface other = moveCommand.item;

		if(this.item != other)
			return false;

		newPos = other.pos();

		return true;
	}
}
