/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.commands;

import com.trolltech.qt.gui.QGraphicsLineItem;
import com.trolltech.qt.gui.QLineF;
import com.trolltech.qt.gui.QUndoCommand;

/**
 * Resize a line
 * 
 * @author Jesse Yates
 */
public class ResizeLineItemCommand extends ResizeItemCommand
{
	/** The item that is being moved */
	QGraphicsLineItem item;

	/**
	 * Constructor
	 * 
	 * @param oldLine
	 *        The previous item position
	 * @param item
	 *        The current item
	 */
	public ResizeLineItemCommand(QLineF oldLine, QGraphicsLineItem line)
	{
		this.oldP1 = oldLine.p1();
		this.oldP2 = oldLine.p2();
		this.newP1 = line.line().p1();
		this.newP2 = line.line().p2();
		this.item = line;
	}

	/**
	 * Move the item back to the previous location
	 */
	@Override
	public void undo()
	{
		item.prepareGeometryChange();
		item.setLine(new QLineF(this.oldP1, this.oldP2));
	}

	/**
	 * Move the item back to the original location
	 */
	@Override
	public void redo()
	{
		item.prepareGeometryChange();
		this.item.setLine(new QLineF(this.newP1, this.newP2));
	}

	/**
	 * Combine two moves if they are the same item over a set of points, with preference to the older location
	 */
	@Override
	public boolean mergeWith(QUndoCommand command)
	{
		System.out.println("Merging Resize");
		ResizeLineItemCommand moveCommand;
		if(command instanceof ResizeLineItemCommand)
			moveCommand = (ResizeLineItemCommand) command;
		else
		{
			System.out.println("Not a line command: false");
			return false;
		}
		QGraphicsLineItem other = moveCommand.item;

		if(this.item != other)
		{
			System.out.println("Other item is different: " + this + " vs. " + other);
			return false;
		}

		this.oldP1 = other.line().p1();
		this.oldP2 = other.line().p2();
		//TODO need to replace this part with the previous/next check combine?

		return true;
	}
}
