/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import java.io.IOException;
import java.io.Serializable;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.QRectF;
import com.trolltech.qt.gui.QGraphicsEllipseItem;
import com.trolltech.qt.gui.QGraphicsItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.MoveItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeEllipseItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeItemCommand;

/**
 * Circle capable of drag and drop
 * 
 * @author Jesse Yates
 */
public class DragDropEllipse extends QGraphicsEllipseItem implements DragDropItem, Serializable
{
	private static final long serialVersionUID = 1L;

	public transient QSignalEmitter.Signal1<MoveItemCommand> moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
	public transient QSignalEmitter.Signal1<ResizeItemCommand> resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();

	private QPointF oldP;
	private QRectF oldRect;
	private boolean clicked;
	private boolean adjustSize;
	private boolean adjustBottom;
	private boolean adjustTop;
	private boolean adjustRight;
	private boolean adjustLeft;

	public DragDropEllipse(QGraphicsEllipseItem current, DrawingGraphicsScene scene)
	{
		this(current);
		this.setScene(scene);
	}

	public DragDropEllipse(QGraphicsEllipseItem current)// , DrawingGraphicsScene scene)
	{
		super(current.rect());
		this.setPen(current.pen());
		setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, true);
		setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, false);
		syncPos();
	}

	/** Sync the old and current positions */
	public void syncPos()
	{
		oldP = this.pos();
	}

	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			if(checkResizeArea(event.scenePos()))
			{
				QPointF p1 = new QPointF((this.boundingRect().width() / 2) + this.boundingRect().left(), this.boundingRect().bottom());
				QPointF p2 = new QPointF((this.boundingRect().width() / 2) + this.boundingRect().left(), this.boundingRect().top());
				QPointF p3 = new QPointF(this.boundingRect().right(), (this.boundingRect().height() / 2) + this.boundingRect().top());
				QPointF p4 = new QPointF(this.boundingRect().left(), (this.boundingRect().height() / 2) + this.boundingRect().top());

				if(DragDropUtilities.checkPoint(p1, event.scenePos(), this))
					setAdjust(true, false, false, false);
				else if(DragDropUtilities.checkPoint(p2, event.scenePos(), this))
					setAdjust(false, true, false, false);
				else if(DragDropUtilities.checkPoint(p3, event.scenePos(), this))
					setAdjust(false, false, true, false);
				else if(DragDropUtilities.checkPoint(p4, event.scenePos(), this))
					setAdjust(false, false, false, true);
				this.adjustSize = true;
				this.oldRect = this.rect().clone();
				DragDropUtilities.setResizeCursor(this);
				// this.mouseP = event.scenePos();
			}
			else
			{
				super.mousePressEvent(event);
				DragDropUtilities.mousePressEvent(event, this, true);
			}
			event.accept();
			return;
		}
		event.ignore();
	}

	private void setAdjust(boolean p1, boolean p2, boolean p3, boolean p4)
	{
		this.adjustBottom = p1;
		this.adjustTop = p2;
		this.adjustRight = p3;
		this.adjustLeft = p4;
	}

	public void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			if(adjustSize)
			{
				if(adjustSize)
				{
					// System.out.println("Adjust Ellipse size");
					DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));
					QRectF newRect = this.rect().clone();

					// this.scale(this.mouseP.x() - event.scenePos().x(), this.mouseP.y() - event.scenePos().y());

					if(adjustTop)
						newRect.setTop(event.scenePos().y());

					else if(adjustLeft)
						newRect.setLeft(event.scenePos().x());

					else if(adjustRight)
						newRect.setRight(event.scenePos().x());

					else if(adjustBottom)
						newRect.setBottom(event.scenePos().y());

					this.prepareGeometryChange();
					this.setRect(newRect);
					// this.adjustSize = false;
				}
			}
			else
			{
				super.mouseMoveEvent(event);
				DragDropUtilities.mouseMoveEvent(event, this, true);
			}
			event.accept();
			return;
		}
		event.ignore();
	}

	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			super.mouseReleaseEvent(event);
			DragDropUtilities.mouseReleaseEvent(event, this, true);
			this.adjustSize = false;
			// event.accept();
			return;
		}
		event.ignore();
	}

	@Override
	public void emitMove()
	{
		this.moveItem.emit(new MoveItemCommand(oldP, this));
	}

	@Override
	public QPointF oldPosition()
	{
		return this.oldP;
	}

	/**
	 * @return the clicked
	 */
	public boolean isClicked()
	{
		return clicked;
	}

	/**
	 * @param clicked
	 *        the clicked to set
	 */
	public void setClicked(boolean clicked)
	{
		this.clicked = clicked;
	}

	/**
	 * Checks to see if the event is close to a need to resize
	 * 
	 * @param eventScene
	 * @return <code>true</code> if it in the distance, <code>false</code> otherwise
	 */
	public boolean checkResizeArea(QPointF eventScene)
	{
		QPointF p1 = new QPointF((this.boundingRect().width() / 2) + this.boundingRect().left(), this.boundingRect().bottom());
		QPointF p2 = new QPointF((this.boundingRect().width() / 2) + this.boundingRect().left(), this.boundingRect().top());
		QPointF p3 = new QPointF(this.boundingRect().right(), (this.boundingRect().height() / 2) + this.boundingRect().top());
		QPointF p4 = new QPointF(this.boundingRect().left(), (this.boundingRect().height() / 2) + this.boundingRect().top());
		QPointF p5 = this.boundingRect().topLeft();

		if(DragDropUtilities.checkPoint(p1, eventScene, this))
			return true;
		if(DragDropUtilities.checkPoint(p2, eventScene, this))
			return true;
		if(DragDropUtilities.checkPoint(p3, eventScene, this))
			return true;
		if(DragDropUtilities.checkPoint(p4, eventScene, this))
			return true;
		if(DragDropUtilities.checkPoint(p5, eventScene, this))
			return true;
		return false;
	}

	public void emitResize()
	{
		if(this.oldRect != null)
			this.resizeItem.emit(new ResizeEllipseItemCommand(oldRect, this));
	}

	@Override
	public boolean checkWasResized()
	{
		if(this.oldRect == null)
			return false;
		else
		{
			// System.out.println("else: rect" + this.oldRect);
			if(DragDropUtilities.checkDistance(this.oldRect.topLeft(), this.rect().topLeft()))
				return true;
			// System.out.println("\t not top-left");
			if(DragDropUtilities.checkDistance(this.oldRect.topRight(), this.rect().topRight()))
				return true;
			// System.out.println("\t not top-right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomRight(), this.rect().bottomRight()))
				return true;
			// System.out.println("\t not bottom-Right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomLeft(), this.rect().bottomLeft()))
				return true;
			// System.out.println("\t not bottom left");
			return false;
		}
	}

	@Override
	public void syncSize()
	{
		this.oldRect = null;

	}

	@Override
	public DragDropItem copy()
	{
		return new DragDropEllipse(this);
	}

	@Override
	public void setScene(DrawingGraphicsScene scene)
	{
		// TODO Abstract this behavior away somehow, but retain the QGraphicsLineItem superclass
		this.moveItem.connect(scene, "moveEvent(MoveItemCommand)");
		this.resizeItem.connect(scene, "resizeEvent(ResizeItemCommand)");

	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		out.writeObject(new double[]{super.rect().x(), super.rect().y(), super.rect().width(), super.rect().height()});
		out.writeObject(new double[]{oldP.x(), oldP.y()});
		// out.writeObject(new double[]{oldRect.x(), oldRect.y(), oldRect.width(), oldRect.height()});
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		double[] ellipse = (double[]) in.readObject();
		double[] point = (double[]) in.readObject();
		// double[] rect = (double[]) in.readObject();

		super.setRect(ellipse[0], ellipse[1], ellipse[2], ellipse[3]);
		this.oldP = new QPointF(point[0], point[1]);
		// this.oldRect = new QRectF(rect[0], rect[1], rect[2], rect[3]);

		moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
		resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();
	}
}
