/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import java.io.IOException;
import java.io.Serializable;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.gui.QGraphicsLineItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QLineF;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.MoveItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeLineItemCommand;

/**
 * Line that can accept drag and drops
 * 
 * @author Jesse Yates
 */
public class DragDropLine extends QGraphicsLineItem implements DragDropItem, Serializable
{
	private static final long serialVersionUID = 1L;

	public QSignalEmitter.Signal1<MoveItemCommand> moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
	public QSignalEmitter.Signal1<ResizeItemCommand> resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();

	private QPointF oldP;
	private QLineF oldLine;
	private boolean adjustSizeP1;
	private boolean adjustSizeP2;
	private boolean adjustSize;
	private boolean clicked;

	public DragDropLine(QGraphicsLineItem line, DrawingGraphicsScene scene)
	{
		this(line);
		setScene(scene);
	}

	public DragDropLine(QGraphicsLineItem line)
	{
		super(line.line());
		this.setPen(line.pen());
		syncPos();
	}

	/** Sync the old and current positions */
	public void syncPos()
	{
		oldP = this.pos();
	}

	public void syncResize()
	{
		this.oldLine = null;
	}

	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Mouse press event: \n\t" + this);

		if(this.isSelected())
		{
			if(checkResizeArea(event.scenePos()))
			{
				this.adjustSize = true;
				if(DragDropUtilities.checkPoint(this.line().p1(), event.scenePos(), this))
				{
					this.adjustSizeP1 = true;
					this.adjustSizeP2 = false;
				}
				if(DragDropUtilities.checkPoint(this.line().p2(), event.scenePos(), this))
				{
					this.adjustSizeP2 = true;
					this.adjustSizeP1 = false;
				}
				this.oldLine = new QLineF(this.line().p1(), this.line().p2());
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));

			}
			else
			{
				System.out.println("Line is selected");
				// super.mousePressEvent(event);
				DragDropUtilities.mousePressEvent(event, this, true);
			}
			event.accept();
			return;
		}
		event.ignore();

	}

	public void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			// System.out.println("Line move event: is selected");
			if(adjustSize)
			{
				// System.out.println("\t  Adjusting size!");
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));
				QLineF newLine = this.line();
				if(adjustSizeP1)
					newLine = new QLineF(this.mapFromParent(event.scenePos()), this.line().p2());
				else if(adjustSizeP2)
					newLine = new QLineF(this.line().p1(), this.mapFromParent(event.scenePos()));
				this.prepareGeometryChange();
				this.setLine(newLine);

				event.accept();
			}
			else
			{
				super.mouseMoveEvent(event);
				DragDropUtilities.mouseMoveEvent(event, this, true);
			}

			event.accept();
			return;

		}
		event.ignore();
	}

	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{

		// System.out.println(this + " recived mouse release");
		if(this.isSelected())
		{
			DragDropUtilities.mouseReleaseEvent(event, this, true);
			this.adjustSize = false;
			event.accept();
			// super.mouseReleaseEvent(event);
			return;
		}
		event.ignore();

	}

	public void mouseDoubleClickEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("DragDrop double click");
		super.mouseDoubleClickEvent(event);
		// DragDropUtilities.mouseDoubleClickEvent(event, this);
	}

	/**
	 * Checks to see if the event is close to either end
	 * 
	 * @param eventScene
	 * @return <code>true</code> if it in the distance, <code>false</code> otherwise
	 */
	public boolean checkResizeArea(QPointF eventScene)
	{
		if(DragDropUtilities.checkPoint(this.line().p1(), eventScene, this) || DragDropUtilities.checkPoint(this.line().p2(), eventScene, this))
			return true;
		return false;
	}

	@Override
	public void emitMove()
	{
		this.moveItem.emit(new MoveItemCommand(oldP, this));

	}

	public void emitResize()
	{
		if(this.oldLine != null)
			this.resizeItem.emit(new ResizeLineItemCommand(oldLine, this));
	}

	@Override
	public QPointF oldPosition()
	{
		return oldP;
	}

	/**
	 * @return the clicked
	 */
	public boolean isClicked()
	{
		return clicked;
	}

	/**
	 * @param clicked
	 *        the clicked to set
	 */
	public void setClicked(boolean clicked)
	{
		this.clicked = clicked;
	}

	@Override
	public boolean checkWasResized()
	{
		if(this.oldLine == null)
			return false;
		else
		{
			if(DragDropUtilities.checkDistance(this.oldLine.p1(), this.line().p1()))
				return true;
			if(DragDropUtilities.checkDistance(this.oldLine.p2(), this.line().p2()))
				return true;
			return false;
		}
	}

	@Override
	public void syncSize()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public DragDropItem copy()
	{
		return new DragDropLine(this);
	}

	@Override
	public void setScene(DrawingGraphicsScene scene)
	{
		// TODO Abstract this behavior away somehow, but retain the Qgraphicslineitem superclass
		this.moveItem.connect(scene, "moveEvent(MoveItemCommand)");
		this.resizeItem.connect(scene, "resizeEvent(ResizeItemCommand)");

	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		QLineF line = super.line();
		out.writeObject(new double[]{line.x1(), line.y1(), line.x2(), line.y2()});
		out.writeObject(new double[]{oldP.x(), oldP.y()});
		// out.writeObject(new double[]{oldRect.x(), oldRect.y(), oldRect.width(), oldRect.height()});
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		double[] line = (double[]) in.readObject();
		double[] point = (double[]) in.readObject();
		// double[] rect = (double[]) in.readObject();

		super.setLine(line[0], line[1], line[2], line[3]);
		this.oldP = new QPointF(point[0], point[1]);
		super.setPos(this.oldP);
		// this.oldRect = new QRectF(rect[0], rect[1], rect[2], rect[3]);

		moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
		resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();
	}
}
