/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import java.io.Serializable;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.QRectF;
import com.trolltech.qt.core.Qt.TransformationMode;
import com.trolltech.qt.gui.QGraphicsItem;
import com.trolltech.qt.gui.QGraphicsPixmapItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QPen;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.MoveItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeItemCommand;

/**
 * @author Jesse Yates
 */
public class DragDropPixmap extends QGraphicsPixmapItem implements DragDropItem, Serializable
{
	private static final long serialVersionUID = 1L;

	public QSignalEmitter.Signal1<MoveItemCommand> moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
	public QSignalEmitter.Signal1<ResizeItemCommand> resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();

	public static final double scaleBy = .98;

	private QRectF oldRect;
	private QPointF oldP;
	private QPointF oldMouse;
	private boolean clicked;
	private boolean adjustSize;
	private boolean adjustSizeTL;
	private boolean adjustSizeTR;
	private boolean adjustSizeBL;
	private boolean adjustSizeBR;

	public DragDropPixmap(QGraphicsPixmapItem map, DrawingGraphicsScene scene)
	{
		this(map);
		setScene(scene);
	}

	public DragDropPixmap(QGraphicsPixmapItem map)
	{
		super(map.pixmap());
		this.setPos(map.pos());
		this.setFlags(map.flags());

		this.setTransformationMode(TransformationMode.FastTransformation);
		setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		syncPos();
	}

	private void setAdjust(boolean tr, boolean tl, boolean br, boolean bl)
	{
		this.adjustSizeBR = br;
		this.adjustSizeBL = bl;
		this.adjustSizeTL = tl;
		this.adjustSizeTR = tr;
	}

	/** Sync the old and current positions */
	public void syncPos()
	{
		oldP = this.pos();
	}

	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{

			if(checkResizeArea(event.scenePos()))
			{
				// System.out.println("Adjust line size");
				this.adjustSize = true;
				if(DragDropUtilities.checkPoint(this.boundingRect().topRight(), event.scenePos(), this))
				{
					this.setAdjust(true, false, false, false);
				}
				else if(DragDropUtilities.checkPoint(this.boundingRect().topLeft(), event.scenePos(), this))
				{
					this.setAdjust(false, true, false, false);
				}
				else if(DragDropUtilities.checkPoint(this.boundingRect().bottomRight(), event.scenePos(), this))
				{
					this.setAdjust(false, false, true, false);
				}
				else
				{
					this.setAdjust(false, false, false, true);
				}
				this.oldRect = this.sceneBoundingRect();
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));
				this.oldMouse = event.scenePos();
			}
			else
			{
				super.mousePressEvent(event);
				DragDropUtilities.mousePressEvent(event, this, true);
			}
		}
	}

	public void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			if(adjustSize)
			{
				// System.out.println("Adjust Pixmap size");
				// TODO attempt to change the scaling parameters to minimize change of lag-crashing
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));
				QPoint difference = this.oldMouse.toPoint().subtract(event.scenePos().toPoint());
				if(DragDropUtilities.checkDistance(this.oldMouse, event.scenePos()))
				{
					double directionX = (1 - scaleBy) * 2;
					double directionY = (1 - scaleBy) * 2;
					// System.out.println("Difference: " + difference);
					// System.out.println("Mouse distance: " + difference.manhattanLength());
					this.prepareGeometryChange();

					if(adjustSizeTR)
					{
						if(difference.x() > 0)
							directionX = 0;
						if(difference.y() < 0)
							directionY = 0;
						this.setPos(new QPointF(this.pos().x(), event.scenePos().y()));
					}

					else if(adjustSizeTL)
					{
						if(difference.x() < 0)
							directionX = 0;
						if(difference.y() < 0)
							directionY = 0;
						this.setPos(event.scenePos());
					}
					else if(adjustSizeBR)
					{
						if(difference.x() > 0)
							directionX = 0;
						if(difference.y() > 0)
							directionY = 0;
					}
					else if(adjustSizeBL)
					{
						if(difference.x() < 0)
							directionX = 0;
						if(difference.y() > 0)
							directionY = 0;
						this.setPos(new QPointF(event.scenePos().x(), this.pos().y()));
					}
					if(difference.x() == 0)
					{
						// System.out.println("X = 0");
						// System.out.println("scaling: x:" + (scaleBy + directionX) * Math.abs(difference.x()) +
						// ", y: " + (scaleBy + directionY) * Math.abs(difference.x()));
						this.scale(1, (scaleBy + directionY) * Math.abs(difference.y()));
					}
					else if(difference.y() == 0)
					{
						// System.out.println("Y = 0");
						// System.out.println("scaling: x:" + (scaleBy + directionX) * Math.abs(difference.x()) +
						// ", y: " + (scaleBy + directionY) * Math.abs(difference.x()));
						this.scale((scaleBy + directionX), 1);
					}
					else
					{
						// System.out.println("Neither  == 0");
						// System.out.println("scaling: x:" + (scaleBy + directionX) + ", y: " + (scaleBy +
						// directionY));
						this.scale((scaleBy + directionX) * Math.abs(difference.x()), (scaleBy + directionY) * Math.abs(difference.y()));
					}

					this.oldMouse = event.scenePos();
				}
			}
			else
			{
				super.mouseMoveEvent(event);
				DragDropUtilities.mouseMoveEvent(event, this, true);
			}
			event.accept();
			return;
		}
		event.ignore();
	}

	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			DragDropUtilities.mouseReleaseEvent(event, this, true);
			this.adjustSize = false;
			event.accept();
			return;
		}
		event.ignore();
	}

	@Override
	public void emitMove()
	{
		if(this.oldP != null)
			this.moveItem.emit(new MoveItemCommand(oldP, this));

	}

	@Override
	public QPointF oldPosition()
	{
		return this.oldP;
	}

	@Override
	public void setPen(QPen pen)
	{
		return;
	}

	/**
	 * @return the clicked
	 */
	public boolean isClicked()
	{
		return clicked;
	}

	/**
	 * @param clicked
	 *        the clicked to set
	 */
	public void setClicked(boolean clicked)
	{
		this.clicked = clicked;
	}

	@Override
	public boolean checkResizeArea(QPointF eventScene)
	{
		if(DragDropUtilities.checkPoint(this.boundingRect().topLeft(), eventScene, this) || DragDropUtilities.checkPoint(this.boundingRect().topRight(), eventScene, this) || DragDropUtilities.checkPoint(this.boundingRect().bottomRight(), eventScene, this) || DragDropUtilities.checkPoint(this.boundingRect().bottomLeft(), eventScene, this))
			return true;
		return false;
	}

	// TODO work on getting the resize working for a pixmap
	public void emitResize()
	{
		// this.resizeItem.emit(new ResizePixmapItemCommand(oldRect, oldP, this));
	}

	@Override
	public boolean checkWasResized()
	{
		// System.out.println("Checking Size: Rect");
		if(this.oldRect == null)
			return false;
		else
		{
			System.out.println("else: boundingRect" + this.oldRect);
			System.out.println("this rect: " + this.sceneBoundingRect());
			if(DragDropUtilities.checkDistance(this.oldRect.topLeft(), this.boundingRect().topLeft()))
				return true;
			System.out.println("\t not top-left");
			if(DragDropUtilities.checkDistance(this.oldRect.topRight(), this.boundingRect().topRight()))
				return true;
			System.out.println("\t not top-right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomRight(), this.boundingRect().bottomRight()))
				return true;
			System.out.println("\t not bottom-Right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomLeft(), this.boundingRect().bottomLeft()))
				return true;
			System.out.println("\t not bottom left");
			return false;
		}

	}

	@Override
	public void syncSize()
	{
		this.oldRect = null;

	}

	@Override
	public DragDropItem copy()
	{
		return new DragDropPixmap(this);
	}

	@Override
	public void setScene(DrawingGraphicsScene scene)
	{
		// TODO Abstract this behavior away somehow, but retain the Qgraphicslineitem superclass
		this.moveItem.connect(scene, "moveEvent(MoveItemCommand)");
		this.resizeItem.connect(scene, "resizeEvent(ResizeItemCommand)");

	}
}
