/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import java.io.IOException;
import java.io.Serializable;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.QRectF;
import com.trolltech.qt.gui.QGraphicsItem;
import com.trolltech.qt.gui.QGraphicsRectItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.MoveItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeItemCommand;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.commands.ResizeRectItemCommand;

/**
 * @author Jesse Yates
 */
public class DragDropRect extends QGraphicsRectItem implements DragDropItem, Serializable
{
	private static final long serialVersionUID = 1L;

	public QSignalEmitter.Signal1<MoveItemCommand> moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
	public QSignalEmitter.Signal1<ResizeItemCommand> resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();

	private QPointF oldP;
	private QRectF oldRect;

	private boolean adjustSize;
	private boolean adjustSizeTL;
	private boolean adjustSizeTR;
	private boolean adjustSizeBL;
	private boolean adjustSizeBR;
	private boolean clicked;

	public DragDropRect(QGraphicsRectItem current, DrawingGraphicsScene scene)
	{
		this(current);
		setScene(scene);
	}

	public DragDropRect(QGraphicsRectItem current)
	{
		super(current.rect());
		this.setPen(current.pen());
		setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, false);
		this.setAcceptDrops(false);
		syncPos();
	}

	/** Sync the old and current positions */
	public void syncPos()
	{
		oldP = this.pos();
	}

	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			if(checkResizeArea(event.scenePos()))
			{
				// System.out.println("Adjust line size");
				this.adjustSize = true;
				if(DragDropUtilities.checkPoint(this.rect().topRight(), event.scenePos(), this))
				{
					this.setAdjust(true, false, false, false);
				}
				else if(DragDropUtilities.checkPoint(this.rect().topLeft(), event.scenePos(), this))
				{
					this.setAdjust(false, true, false, false);
				}
				else if(DragDropUtilities.checkPoint(this.rect().bottomRight(), event.scenePos(), this))
				{
					this.setAdjust(false, false, true, false);
				}
				else
				{
					this.setAdjust(false, false, false, true);
				}
				this.oldRect = this.rect();
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));

			}
			else
			{
				super.mousePressEvent(event);
				DragDropUtilities.mousePressEvent(event, this, true);
			}
		}
	}

	private void setAdjust(boolean tr, boolean tl, boolean br, boolean bl)
	{
		this.adjustSizeBR = br;
		this.adjustSizeBL = bl;
		this.adjustSizeTL = tl;
		this.adjustSizeTR = tr;
	}

	public void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			if(adjustSize)
			{
				// System.out.println("Adjust Rect size");
				DragDropUtilities.setResizeCursor(this);// this.setCursor(new QCursor(CursorShape.CrossCursor));
				QRectF newRect = this.rect();

				if(adjustSizeTR)
					newRect.setTopRight(event.scenePos());// = new QRectF(new QPointF(newRect.topLeft().x(),
				// this.mapFromParent(event.scenePos()).y()), new
				// QPointF(this.mapFromParent(event.scenePos()).x(),
				// this.rect().bottomRight().y()));

				else if(adjustSizeTL)
					newRect.setTopLeft(event.scenePos());// = new QRectF(new
				// QPointF(this.mapFromParent(event.scenePos()).x(),
				// this.mapFromParent(event.scenePos()).y()),
				// newRect.bottomRight());

				else if(adjustSizeBR)
					newRect.setBottomRight(event.scenePos());// = new QRectF(newRect.topLeft(), new
				// QPointF(this.mapFromParent(event.scenePos()).x(),
				// this.mapFromParent(event.scenePos()).y()));

				else if(adjustSizeBL)
					newRect.setBottomLeft(event.scenePos());// = new QRectF(new
				// QPointF(this.mapFromParent(event.scenePos()).x(),
				// newRect.topLeft().y()), new
				// QPointF(this.rect().bottomRight().x(),
				// this.mapFromParent(event.scenePos()).y()));
				this.prepareGeometryChange();
				this.setRect(newRect);
				// this.scene().update();
			}
			else
			{
				super.mouseMoveEvent(event);
				DragDropUtilities.mouseMoveEvent(event, this, true);
			}
			event.accept();
			return;
		}
		event.ignore();
	}

	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.isSelected())
		{
			DragDropUtilities.mouseReleaseEvent(event, this, true);
			this.adjustSize = false;
			event.accept();
			return;
		}
		event.ignore();
	}

	@Override
	public void emitMove()
	{
		if(this.oldP != null)
			this.moveItem.emit(new MoveItemCommand(oldP, this));

	}

	@Override
	public QPointF oldPosition()
	{
		return this.oldP;
	}

	/**
	 * @return the clicked
	 */
	public boolean isClicked()
	{
		return clicked;
	}

	/**
	 * @param clicked
	 *        the clicked to set
	 */
	public void setClicked(boolean clicked)
	{
		this.clicked = clicked;
	}

	@Override
	public boolean checkResizeArea(QPointF eventScene)
	{
		if(DragDropUtilities.checkPoint(this.rect().topLeft(), eventScene, this) || DragDropUtilities.checkPoint(this.rect().topRight(), eventScene, this) || DragDropUtilities.checkPoint(this.rect().bottomRight(), eventScene, this) || DragDropUtilities.checkPoint(this.rect().bottomLeft(), eventScene, this))
			return true;
		return false;
	}

	public void emitResize()
	{
		System.out.println("Emit resize rect");
		if(this.oldRect != null)
			this.resizeItem.emit(new ResizeRectItemCommand(oldRect, this));
	}

	@Override
	public boolean checkWasResized()
	{
		// System.out.println("Checking Size: Rect");
		if(this.oldRect == null)
			return false;
		else
		{
			// System.out.println("else: rect" + this.oldRect);
			if(DragDropUtilities.checkDistance(this.oldRect.topLeft(), this.rect().topLeft()))
				return true;
			// System.out.println("\t not top-left");
			if(DragDropUtilities.checkDistance(this.oldRect.topRight(), this.rect().topRight()))
				return true;
			// System.out.println("\t not top-right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomRight(), this.rect().bottomRight()))
				return true;
			// System.out.println("\t not bottom-Right");
			if(DragDropUtilities.checkDistance(this.oldRect.bottomLeft(), this.rect().bottomLeft()))
				return true;
			// System.out.println("\t not bottom left");
			return false;
		}
	}

	@Override
	public void syncSize()
	{
		this.oldRect = null;

	}

	@Override
	public DragDropItem copy()
	{
		return new DragDropRect(this);
	}

	@Override
	public void setScene(DrawingGraphicsScene scene)
	{
		// TODO Abstract this behavior away somehow, but retain the Qgraphicslineitem superclass
		this.moveItem.connect(scene, "moveEvent(MoveItemCommand)");
		this.resizeItem.connect(scene, "resizeEvent(ResizeItemCommand)");

	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		out.writeObject(new double[]{super.rect().x(), super.rect().y(), super.rect().width(), super.rect().height()});
		out.writeObject(new double[]{oldP.x(), oldP.y()});
		// out.writeObject(new double[]{oldRect.x(), oldRect.y(), oldRect.width(), oldRect.height()});
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		double[] rect = (double[]) in.readObject();
		double[] point = (double[]) in.readObject();
		// double[] rect = (double[]) in.readObject();

		super.setRect(rect[0], rect[1], rect[2], rect[3]);
		this.oldP = new QPointF(point[0], point[1]);
		// this.oldRect = new QRectF(rect[0], rect[1], rect[2], rect[3]);

		moveItem = new QSignalEmitter.Signal1<MoveItemCommand>();
		resizeItem = new QSignalEmitter.Signal1<ResizeItemCommand>();
	}
}
