/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.Qt.CursorShape;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QCursor;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QGraphicsItem.GraphicsItemFlag;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;

/**
 * General DragDrop utilities to handle the general drag&drop issues/mouse events/cursor selection
 * 
 * @author Jesse Yates
 */
public class DragDropUtilities
{
	public static final int MinDistanceAdjust = 4;

	/* (non-Javadoc)
	 * @see edu.jhu.cs.oose.elevel.AdvancedEditCard.DragDropItem#mousePressEvent(com.trolltech.qt.gui.QGraphicsSceneMouseEvent)
	 */
	public static void mousePressEvent(QGraphicsSceneMouseEvent event, DragDropItem item, boolean useDefaultCursor)
	{
		item.setFlag(GraphicsItemFlag.ItemIsMovable, true);
		item.setFlag(GraphicsItemFlag.ItemIsSelectable, true);

		//item.setSelected(true);
		item.update(item.boundingRect());
		item.syncPos();
		item.setClicked(true);
		if(useDefaultCursor)
			DragDropUtilities.setDragCursor(item);//item.setCursor(new QCursor(CursorShape.ClosedHandCursor));
		//System.out.println("dragdrop: press mouse: " + item);
	}

	/* (non-Javadoc)
	 * @see edu.jhu.cs.oose.elevel.AdvancedEditCard.DragDropItem#mouseMoveEvent(com.trolltech.qt.gui.QGraphicsSceneMouseEvent)
	 */

	public static void mouseMoveEvent(QGraphicsSceneMouseEvent event, DragDropItem item, boolean useDefaultCursor)
	{

		//System.out.println("DragDrop move: " + item);
		//item.setFlag(GraphicsItemFlag.ItemIsMovable, true);
		//
		//done to ensure proper cursor selection

		if(useDefaultCursor)
		{
			if(item.isClicked())
			{
				if(item.cursor().shape().compareTo(CursorShape.ClosedHandCursor) != 0)
					DragDropUtilities.setDragCursor(item);//item.setCursor(new QCursor(CursorShape.ClosedHandCursor));
				//System.out.println("Closed hand!");
			}
			else if(item.checkResizeArea(event.scenePos()))
			{
				DragDropUtilities.setResizeCursor(item);//item.setCursor(new QCursor(CursorShape.CrossCursor));
				//System.out.println("\t Close to end point");
			}
			else
			{
				if(item.cursor().shape().compareTo(CursorShape.OpenHandCursor) != 0)
					DragDropUtilities.setReleaseCursor(item);//item.setCursor(new QCursor(CursorShape.OpenHandCursor));
				//System.out.println("Open hand!");
			}
		}
		item.update(item.boundingRect());
	}

	/* (non-Javadoc)
	 * @see edu.jhu.cs.oose.elevel.AdvancedEditCard.DragDropItem#mouseReleaseEvent(com.trolltech.qt.gui.QGraphicsSceneMouseEvent)
	 */

	public static void mouseReleaseEvent(QGraphicsSceneMouseEvent event, DragDropItem item, boolean useDefaultCursor)
	{
		if(item.oldPosition().toPoint().subtract(item.pos().toPoint()).manhattanLength() > QApplication.startDragDistance())
		{
			System.out.println("Adding move!");
			item.emitMove();
			item.syncPos();
		}
		if(item.checkWasResized())
		{
			System.out.println("Adding resize");
			item.emitResize();
			item.syncSize();
		}

		item.setFlag(GraphicsItemFlag.ItemIsMovable, false);

		if(useDefaultCursor)
		{
			if(item.checkResizeArea(event.scenePos()))
				DragDropUtilities.setResizeCursor(item);//item.setCursor(new QCursor(CursorShape.CrossCursor));
			else
				DragDropUtilities.setReleaseCursor(item);//item.setCursor(new QCursor(CursorShape.OpenHandCursor));
		}
		item.setClicked(false);
		//System.out.println("dragdrop: release mouse" + item);

	}

	public static void mouseDoubleClickEvent(QGraphicsSceneMouseEvent event, DragDropItem item)
	{
		item.setFlag(GraphicsItemFlag.ItemIsMovable, true);
		//item.setFlag(GraphicsItemFlag.ItemIsSelectable, true);

		//item.syncPos();
		item.update(item.boundingRect());

		System.out.println("dragdrop: press mouse" + item);
	}

	/**
	 * Set the item's cursor to a cross cursor
	 * 
	 * @param item
	 *        The <code>DragDropItem</code> to set
	 */
	public static void setResizeCursor(DragDropItem item)
	{
		item.setCursor(new QCursor(CursorShape.CrossCursor));
	}

	/**
	 * Set the item's cursor to a closed hand cursor
	 * 
	 * @param item
	 *        The <code>DragDropItem</code> to set
	 */
	public static void setDragCursor(DragDropItem item)
	{
		item.setCursor(new QCursor(CursorShape.ClosedHandCursor));
	}

	/**
	 * Set the item's cursor to a open hand cursor
	 * 
	 * @param item
	 *        The <code>DragDropItem</code> to set
	 */
	public static void setReleaseCursor(DragDropItem item)
	{
		item.setCursor(new QCursor(CursorShape.OpenHandCursor));
	}

	/**
	 * Checks to see if the mouse event is less than the <code>minAdjustDistance</code> away from the sent
	 * <code>QPointF</code>
	 * 
	 * @param p1
	 *        The comparison point
	 * @param event
	 *        the mouse event
	 * @return <code>true</code> if it in the distance, <code>false</code> otherwise
	 */
	public static boolean checkPoint(QPointF p1, QPointF scenePos, QGraphicsItemInterface item)
	{
		if(p1.toPoint().subtract(item.mapFromParent(scenePos).toPoint()).manhattanLength() < MinDistanceAdjust)
			return true;
		return false;

	}

	public static void printScene(String string, DrawingGraphicsScene scene)
	{
		System.out.println("------------");
		System.out.println(string + " Current graphics scene: " + scene);
		System.out.println("has items:");
		for(QGraphicsItemInterface i : scene.items())
		{
			System.out.println(i);
		}
		System.out.println("selected items:");
		for(QGraphicsItemInterface i : scene.selectedItems())
		{
			System.out.println(i);
		}
		System.out.println("Scene size: " + scene.sceneRect());
		System.out.println("------------");

	}

	public static boolean checkDistance(QPointF p1, QPointF p2)
	{
		QPoint difference = p1.toPoint().subtract(p2.toPoint());

		if(difference.manhattanLength() > 0)
			return true;
		return false;
	}
}
