/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop;

import java.io.IOException;
import java.io.Serializable;
import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QPointF;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QFocusEvent;
import com.trolltech.qt.gui.QGraphicsItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QGraphicsTextItem;
import com.trolltech.qt.gui.QPen;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;

/**
 * @author Jesse Yates
 */
public class FocusableTextItem extends QGraphicsTextItem implements DragDropItem, Serializable
{
	private static final long serialVersionUID = 1L;

	/**
	 * Signal to indicate that it is the item that has lostFocus
	 */
	public transient Signal1<FocusableTextItem> lostFocus = new QSignalEmitter.Signal1<FocusableTextItem>();

	public FocusableTextItem(String string)
	{
		super(string);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, true);
	}

	public FocusableTextItem(FocusableTextItem item)
	{
		super(item.toPlainText());
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, true);
		this.setPos(item.pos());
	}

	/**
	 * Called when the item has gained focus - output to console
	 */
	public void focusInEvent(QFocusEvent event)
	{
		System.out.println("Gained focus!" + this);
		super.focusInEvent(event);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		// this.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction);
		// super.focusInEvent(event);
	}

	/**
	 * Called when the item has lost focus - emit the <code>lostFocus</code> signal
	 */
	public void focusOutEvent(QFocusEvent event)
	{
		System.out.println("LostFocus: " + this);
		this.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, false);
		this.setSelected(false);
		lostFocus.emit(this);
		// super.focusOutEvent(event);
	}

	/**
	 * Handles the mousePressEvent that occurs on the object
	 */
	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("\t\tText mouse click!");
		super.mousePressEvent(event);
		super.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction);
		this.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		event.accept();
	}

	@Override
	public boolean checkResizeArea(QPointF eventScene)
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean checkWasResized()
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public DragDropItem copy()
	{
		return new FocusableTextItem(this);
	}

	@Override
	public void emitMove()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public void emitResize()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public boolean isClicked()
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public QPointF oldPosition()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setClicked(boolean click)
	{
		// TODO Auto-generated method stub

	}

	@Override
	public void setPen(QPen pen)
	{
		// TODO Auto-generated method stub

	}

	@Override
	public void syncPos()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public void syncSize()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public void setScene(DrawingGraphicsScene scene)
	{
		this.lostFocus.connect(scene, "itemLostFocus(QGraphicsTextItem)");

	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		out.writeObject(new double[]{this.pos().x(), this.pos().y()});
		out.writeObject(super.toPlainText());
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		double[] rect = (double[]) in.readObject();
		String text = (String) in.readObject();

		this.setPos(rect[0], rect[1]);
		this.setPlainText(text);

		this.lostFocus = new QSignalEmitter.Signal1<FocusableTextItem>();
	}

}
