/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsRectItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QPainterPath;
import com.trolltech.qt.gui.QPen;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;

/**
 * @author Jesse Yates
 */
public class DrawingSelectItem extends DrawingRectItem
{
	protected static final QPen StandardSelectionPen = new QPen(QColor.darkGray, 2, Qt.PenStyle.DashLine, Qt.PenCapStyle.FlatCap, Qt.PenJoinStyle.MiterJoin);
	private boolean selectedItem;

	public DrawingSelectItem(DrawingGraphicsScene scene)
	{
		super(scene);
		this.selectedItem = false;
	}

	public DrawingSelectItem()
	{
		this(new DrawingGraphicsScene());
	}

	public void setScene(DrawingGraphicsScene scene)
	{
		super.setScene(scene);
		this.scene.setSelectMode(true);
	}

	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Select Press event");
		//DragDropUtilities.printScene("select item", this.scene);

		//if there are no items selected, select the item at that point
		if(this.scene().selectedItems().size() == 0)
		{
			if(this.scene().itemAt(event.scenePos()) != null)
			{
				System.out.println("\t is is not null, setting selected");
				this.scene().itemAt(event.scenePos()).setSelected(true);//.mouseMoveEvent(event);// setFocus(Qt.FocusReason.MouseFocusReason);
				this.scene().update();
				for(QGraphicsItemInterface i : this.scene().selectedItems())
				{
					System.out.println("\t\t selected item: " + i);
					//i.mousePressEvent(event);
				}
			}
			//the item is null, but want might to select multiple items
			else
			{
				System.out.println("Creating new select square");
				this.drawingSquare = new QGraphicsRectItem(event.scenePos().x(), event.scenePos().y(), 0, 0);
				this.drawingSquare.setPen(DrawingSelectItem.StandardSelectionPen);
				this.scene.addItem(drawingSquare);
				this.selectedItem = false;
			}

		}
		else
		//there are some items select
		{
			//if there is no item under the mouse, unselect all the items
			if(this.scene().itemAt(event.scenePos()) == null)
			{
				for(QGraphicsItemInterface i : this.scene.selectedItems())
					i.setSelected(false);
			}
			if(this.scene().itemAt(event.scenePos()) != null)
			{
				//if there is only 1 item selected, unselect the selected items
				if(this.scene().selectedItems().size() == 1)
				{
					for(QGraphicsItemInterface i : this.scene.selectedItems())
						i.setSelected(false);
				}
				//and then select the item under the cursor
				//maintains the selected items as selected
				this.scene().itemAt(event.scenePos()).setSelected(true);

			}
			this.selectedItem = true;
			this.drawingSquare = null;
			//	event.ignore();
		}
		event.accept();
		return;

	}

	/**
	 * Adds the square that is currently being drawn to the stack and draws the element
	 * 
	 * @param event
	 *        The event causing this action
	 */
	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.drawingSquare != null && !selectedItem)
		{
			this.scene.removeItem(this.drawingSquare);
			QPainterPath path = new QPainterPath();
			path.addRect(this.drawingSquare.rect());
			this.scene.setSelectionArea(path, Qt.ItemSelectionMode.ContainsItemBoundingRect);
			//DragDropUtilities.printScene("select item", this.scene);
			event.accept();
			System.out.println("Mouse release select");
			this.drawingSquare.removeFromIndex();
		}
		else
			event.ignore();
		this.drawingSquare = null;
		this.selectedItem = false;

	}

	@Override
	public DrawingItem copy(DrawingGraphicsScene scene)
	{
		DrawingSelectItem ret = new DrawingSelectItem(scene);
		return ret;
	}

}
