/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard.drawItems;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QGraphicsItem;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.dragDrop.FocusableTextItem;

/**
 * @author Jesse Yates
 */
public class DrawingTextItem extends DrawingItem
{
	private FocusableTextItem text;

	/**
	 * Constructor that creates item with sent <code>String</code>
	 * 
	 * @param string
	 *        The string to initialize the element with
	 */
	public DrawingTextItem(DrawingGraphicsScene scene, String string)
	{
		super(scene);
		text = new FocusableTextItem(string);
		text.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, true);
		text.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, true);
		scene.addItem(text);
	}

	public DrawingTextItem(DrawingGraphicsScene scene)
	{
		super(scene);
	}

	public DrawingTextItem()
	{
		super(new DrawingGraphicsScene());
	}

	/**
	 * If there is already an object at the current location, does not do anything, passes on the mouse click to that
	 * element. Otherwise, creates a textbox at that location
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	public void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("TextPressEvent");
		System.out.println("Scene:" + this.scene);
		// if(this.scene.itemAt(event.scenePos()) == null)
		{
			System.out.println("Item is null!");
			text = new FocusableTextItem("");
			text.setFont(scene.font());
			text.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction);
			text.setScene(scene);
			// text.setEnabled(true);
			// text.setZValue(1000.0);

			if(!this.scene.isCenterText())
				text.setPos(event.scenePos());
			else
				text.setPos(0, 0);

			System.out.println("Adding text at: " + text.pos());
			this.eventItem.emit(text);
			event.accept();
		}
		/*
		 * else
		 * {
		 * System.out.println("Item is not null!");
		 * //this.scene.itemAt(event.scenePos()).mousePressEvent(event);
		 * }
		 */
	}

	@Override
	public void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		// System.out.println("Mouse move event!");

	}

	@Override
	public void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Mouse release event!");
		System.out.println("Text:" + text);

	}

	public String toString()
	{
		StringBuffer out = new StringBuffer();
		out.append("Text:");
		if(text != null)
			out.append(text.toPlainText());
		out.append(",loc:");
		if(text != null)
			out.append(text.scenePos());
		return out.toString();
	}

	public void buildFromString(String s)
	{
		System.out.println("Building from: " + s);
		text.setPlainText(s.substring(s.indexOf(":") + 1, s.indexOf(",")));
		s = s.substring(s.indexOf("(") + 1, s.indexOf(")"));
		System.out.println("s=" + s);
		Double x = new Double(s.substring(0, s.indexOf(",")));
		Double y = new Double(s.substring(s.indexOf(",") + 2));
		text.setPos(x, y);

	}

	@Override
	public DrawingItem copy(DrawingGraphicsScene scene)
	{
		DrawingTextItem ret = new DrawingTextItem(scene);
		return ret;
	}

}
