/**
 * CardAreaGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 20, 2009
 */
package edu.jhu.cs.oose.elevel;

import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QImage;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QWidget;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsView;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * Uses the basic interface class QWidget to design the <code>Card</code> GUI
 * 
 * @author Pablo Lee
 */
public class CardAreaGUI extends QWidget
{
	private ViewState state;

	/** The QLayout of the card area */
	private QGridLayout gridLayout;

	/** An image of the logo */
	private QLabel logoImage;

	/**
	 * Initializes the basic layout of a <code>Card</code>, including previous and next card buttons
	 * 
	 * @param window
	 *        the <code>MainWindow</code> of E-Level
	 */
	public CardAreaGUI(QMainWindow window)
	{
		state = ViewState.INSTANCE;
		state.cardSelectedInTree.connect(this, "displayCard(Card)");

		this.gridLayout = new QGridLayout(this);
		setLayout(this.gridLayout);

		QPushButton previousButton = new QPushButton(tr("Previous"), this);
		previousButton.clicked.connect(state, "previousCard()");
		this.gridLayout.addWidget(previousButton, 2, 0, Qt.AlignmentFlag.AlignLeft);

		QPushButton nextButton = new QPushButton(tr("Next"), this);
		nextButton.clicked.connect(state, "nextCard()");
		this.gridLayout.addWidget(nextButton, 2, 1, Qt.AlignmentFlag.AlignRight);

		QLabel questionLabel = new QLabel("Question");
		this.gridLayout.addWidget(questionLabel, 0, 0, Qt.AlignmentFlag.AlignCenter);

		QLabel answerLabel = new QLabel("Answer");
		this.gridLayout.addWidget(answerLabel, 0, 1, Qt.AlignmentFlag.AlignCenter);

		QLabel additionalInfoLabel = new QLabel("Additional Information");
		this.gridLayout.addWidget(additionalInfoLabel, 3, 0, Qt.AlignmentFlag.AlignCenter);

		QLabel hintLabel = new QLabel("Hint");
		this.gridLayout.addWidget(hintLabel, 3, 1, Qt.AlignmentFlag.AlignCenter);

		logoImage = new QLabel();
		QImage logo = new QImage("classpath:../images/splash.png");
		this.logoImage.setPixmap(QPixmap.fromImage(logo));
		this.gridLayout.addWidget(logoImage, 0, 0, 5, 2, Qt.AlignmentFlag.AlignCenter);
		logoImage.setHidden(true);

		if(state.getNumDecks() == 0)
		{
			displayLogo(true);
		}
		else
		{
			displayCard(state.getCurrentCard());
		}
	}

	/**
	 * Displays the current card
	 * 
	 * @param currentCard
	 *        the <code>Card</code> to be displayed in the card area
	 */
	private void displayCard(Card currentCard)
	{
		if(currentCard != null)
		{
			displayLogo(false);

			CardSection question = currentCard.getQuestion();
			CardSection answer = currentCard.getAnswer();
			CardSection hint = currentCard.getHint();
			CardSection additionalInfo = currentCard.getAdditionalInfo();

			if(question.hasText())
			{
				QTextEdit questionBox = new QTextEdit();
				questionBox.setText(question.getText());
				questionBox.setReadOnly(true);
				questionBox.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(questionBox, 1, 0);
			}
			else
			{
				QLabel q = new QLabel();
				q.setPixmap(new DrawingGraphicsView(question.getScene()).getImage());
				q.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(q, 1, 0);
			}

			if(answer.hasText())
			{
				QTextEdit answerBox = new QTextEdit();
				answerBox.setText(answer.getText());
				answerBox.setReadOnly(true);
				answerBox.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(answerBox, 1, 1);
			}
			else
			{
				QLabel a = new QLabel();
				a.setPixmap(new DrawingGraphicsView(answer.getScene()).getImage());
				a.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(a, 1, 1);
			}

			if(additionalInfo.hasText())
			{
				QTextEdit additionalInfoBox = new QTextEdit();
				additionalInfoBox.setText(additionalInfo.getText());
				additionalInfoBox.setReadOnly(true);
				additionalInfoBox.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(additionalInfoBox, 4, 0);
			}
			else
			{
				QLabel a = new QLabel();
				a.setPixmap(new DrawingGraphicsView(additionalInfo.getScene()).getImage());
				a.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(a, 4, 0);
			}

			if(hint.hasText())
			{
				QTextEdit hintBox = new QTextEdit();
				hintBox.setText(hint.getText());
				hintBox.setReadOnly(true);
				hintBox.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(hintBox, 4, 1);
			}
			else
			{
				QLabel h = new QLabel();
				h.setPixmap(new DrawingGraphicsView(hint.getScene()).getImage());
				h.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
				this.gridLayout.addWidget(h, 4, 1);
			}
		}
		else
		{
			displayLogo(true);
		}
	}

	/**
	 * Method to display/hide logo
	 * 
	 * @param show
	 *        Whether or not you want to show the logo
	 */
	private void displayLogo(boolean show)
	{
		for(int i = 0; i < this.gridLayout.count(); ++i)
			this.gridLayout.itemAt(i).widget().setHidden(show);

		logoImage.setHidden(!show);
	}
}
