/**
 * DeckDialog.java
 * Group 7, Object Oriented Software Engineering
 * Oct 25, 2009
 */
package edu.jhu.cs.oose.elevel;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QDialogButtonBox;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLayout;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QWidget;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.preferences.Preferences;

/**
 * Implements a dialog for creating a new <code>Deck</code>
 * 
 * @author Jesse Yates
 */
public class DeckDialog extends QDialog
{
	/** Name label */
	private QLabel name;

	/** Takes the name of the new deck */
	private QLineEdit nameEdit;
	/** The set of possible buttons */
	private QDialogButtonBox buttonBox;

	/** Possible buttons */
	private QPushButton okButton;
	private QPushButton detailsButton;
	private QPushButton cancelButton;
	private QPushButton export;

	/** The extendible area */
	private QWidget extension;

	/** The directory to save the deck */
	private String exportDir;

	/** signal to indicate it is closing */
	public final QSignalEmitter.Signal1<Deck> closing = new QSignalEmitter.Signal1<Deck>();

	/**
	 * Creates all buttons necessary for the <code>Deck</code> and connects them to the appropriate signal
	 * 
	 * @param parent
	 *        A <code>QWidget</code> representing the parent <code>QWidget</code>
	 */
	public DeckDialog(QWidget parent)
	{
		this.extension = new QWidget();
		this.exportDir = (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION");

		this.setupButtonsLabels();

		QGridLayout extensionLayout = new QGridLayout();
		extensionLayout.setMargin(0);
		extensionLayout.addWidget(this.export, 0, 0, 1, 1);
		extensionLayout.addWidget(new QWidget(), 0, 1);
		extensionLayout.setColumnMinimumWidth(0, this.export.size().width());

		this.extension.setLayout(extensionLayout);

		QHBoxLayout topLeftLayout = new QHBoxLayout();
		topLeftLayout.addWidget(this.name);
		topLeftLayout.addSpacing(3);
		topLeftLayout.addWidget(this.nameEdit);

		QGridLayout leftLayout = new QGridLayout();
		leftLayout.addItem(topLeftLayout, 0, 0, 1, 2);
		leftLayout.addWidget(this.detailsButton, 1, 0, 1, 1, Qt.AlignmentFlag.AlignLeft);
		// leftLayout.addWidget(this.detailsButton);
		// leftLayout.addStretch(1);

		QGridLayout mainLayout = new QGridLayout();
		mainLayout.setSizeConstraint(QLayout.SizeConstraint.SetFixedSize);
		mainLayout.addLayout(leftLayout, 0, 0);
		mainLayout.addWidget(this.buttonBox, 0, 1);
		// mainLayout.addWidget(this.detailsButton, 1, 0);
		// mainLayout.setColumnMinimumWidth(0, this.detailsButton.size().width());
		mainLayout.addWidget(this.extension, 2, 0, 1, 2);
		setLayout(mainLayout);

		setWindowTitle(tr("New Deck"));
		this.extension.hide();
	}

	private void setupButtonsLabels()
	{
		this.name = new QLabel(tr("Name: "));
		this.nameEdit = new QLineEdit();
		this.name.setBuddy(this.nameEdit);

		this.okButton = new QPushButton(tr("&OK"));
		this.okButton.setDefault(true);
		this.okButton.clicked.connect(this, "saveDeck()");

		this.cancelButton = new QPushButton(tr("&Close"));
		this.cancelButton.setCheckable(true);
		this.cancelButton.setAutoDefault(false);
		this.cancelButton.clicked.connect(this, "dontSaveDeck()");

		this.detailsButton = new QPushButton(tr("&Options>>"));
		this.detailsButton.setCheckable(true);
		this.detailsButton.setAutoDefault(false);
		this.detailsButton.toggled.connect(this.extension, "setVisible(boolean)");

		this.buttonBox = new QDialogButtonBox(Qt.Orientation.Vertical);
		this.buttonBox.addButton(this.okButton, QDialogButtonBox.ButtonRole.ActionRole);
		this.buttonBox.addButton(this.cancelButton, QDialogButtonBox.ButtonRole.ActionRole);

		this.export = new QPushButton(tr("Export Location"), this);
		this.export.clicked.connect(this, "changeExportLocation()");
	}

	/**
	 * Sets the export location for this new deck to the location specified by the user in a dialog
	 */
	@SuppressWarnings("unused")
	private void changeExportLocation()
	{
		String initialPath = (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION");
		this.exportDir = QFileDialog.getExistingDirectory(this, this.tr("Deck Export Directory"), initialPath);
		if(!(this.exportDir == null || this.exportDir.equals("")))
		{
			// Assign a new path
			if(!this.exportDir.endsWith("/"))
				this.exportDir = this.exportDir + "/";
		}
		else
			this.exportDir = initialPath;
	}

	public void saveDeck()
	{
		Deck deck = new Deck(this.nameEdit.text());
		deck.setDiskLocation(this.exportDir + "/" + deck.getName() + ".elvl");
		// HPS - write on save, not on creation; screws up serialization for some reason
		// deck.writeToDisk(this, deck.getDiskLocation());
		this.closing.emit(deck);
		this.close();
	}

	public void dontSaveDeck()
	{
		this.close();
	}
}
