/**
 * DefaultCanvas.java
 * The canvas associated with the DefaultToolBar
 * Group 7, Object Oriented Software Engineering
 * Oct 17, 2009
 */
package edu.jhu.cs.oose.elevel.editing;

import com.trolltech.qt.core.QSize;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QTextOption;

/**
 * Sets the defaults for the Canvas, allowing text and the wrapping of text
 * 
 * @author Jesse Yates
 */
public class DefaultCanvas extends QTextEdit implements Canvas
{
	/**
	 * Sets the default canvas to Canvas's settings
	 */
	public DefaultCanvas()
	{
		super();
	}

	/**
	 * when a string is passed in we can then set the canvas to allow
	 * the wrapping of text and set the text to s
	 */
	public DefaultCanvas(String s)
	{
		super();
		this.setWordWrapMode(QTextOption.WrapMode.WordWrap);
		this.setText(s);
	}

	/**
	 * @return the default window sizes
	 */
	public QSize sizeHint()
	{
		return new QSize(DefaultWindowSize.HEIGHT.value(), DefaultWindowSize.WIDTH.value());
	}

	/**
	 * The canvas default is set to print as text
	 * 
	 * @return the string
	 */
	public String toString()
	{
		return this.toPlainText();
	}

	/**
	 * given a string we set the canvas text to it
	 * 
	 * @param question
	 *        the text to be placed in canvas
	 */
	public void setFromString(String question)
	{
		this.setText(question);

	}

}

/**
 * sets the default window sizes and can return the value *
 */
enum DefaultWindowSize
{
	WIDTH(20), HEIGHT(20);

	private final int value;

	DefaultWindowSize(int i)
	{
		this.value = i;
	}

	public int value()
	{
		return this.value;
	}

}
