/**
 * EditCardWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel.editing;

import java.util.Map;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsScene;
import edu.jhu.cs.oose.elevel.dataModel.Card;

/**
 * The underlying model class that handles editing any features of a <code>Card</code>
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class EditCardWindow
{
	/** The <code>Card</code> currently being edited */
	private Card card;

	/** The original version of the <code>Card</code> being edited */
	private Card previous;

	/** The <code>ToolBar</code> currently being used */
	private DefaultToolBar editToolbar;

	/**
	 * Constructor for the editCard model, creates the default {@link ToolBar} to be used
	 * 
	 * @param c
	 *        the {@link Card} currently selected
	 */
	public EditCardWindow(Card c)
	{
		setCard(c);
		setPreviousCard(c);
		this.editToolbar = new DefaultToolBar();
	}

	/**
	 * Constructor for the editCard model
	 * 
	 * @param c
	 *        the <code>Card</code> to be edited
	 * @param currentToolbar
	 *        the <code>ToolBar</code> to edit with
	 */
	public EditCardWindow(Card c, ToolBar currentToolbar)
	{
		this(c);
		// TODO change this line to a different toolbar
	}

	/**
	 * Returns the <code>Card</code> currently being edited
	 * 
	 * @return a {@link Card} representing the card currently being edited
	 */
	public Card getCard()
	{
		return card;
	}

	/**
	 * Set the <code>Card</code> being currently edited to the specified {@link Card}
	 * 
	 * @param c
	 *        a <code>Card</code> representing the card currently being edited
	 */
	public void setCard(Card c)
	{
		if(c == null)
			this.card = new Card();
		else
			this.card = c;
	}

	public void setPreviousCard(Card c)
	{
		if(c != null)
			this.previous = new Card(c.getQuestion(), c.getAnswer(), c.getHint(), c.getAdditionalInfo());
	}

	public Card getPreviousCard()
	{
		return previous;
	}

	/**
	 * Return the {@link ToolBar} currently associated with this <code>EditCardWindow</code>
	 * 
	 * @return the {@link ToolBar} currently associated with this <code>EditCardWindow</code>
	 */
	public DefaultToolBar getToolbar()
	{
		return editToolbar;
	}

	/**
	 * Set the {@link ToolBar} currently being used to the specified {@link ToolBar}
	 * 
	 * @param e
	 *        the {@link ToolBar} to associate with this <code>EditCardWindow</code>
	 */
	public void setToolbar(DefaultToolBar e)
	{
		this.editToolbar = e;
	}

	/**
	 * Saves a new card to the deck with the properties passed in
	 * 
	 * @param question
	 *        the question on the flash card
	 * @param answer
	 *        the answer to the question on the flash card
	 * @param hint
	 *        the user specified hint to the answer
	 * @param addinfo
	 *        any additional information given
	 * @param title
	 *        the title of the card
	 * @param pref
	 *        preferences
	 */
	public void save(DrawingGraphicsScene q, DrawingGraphicsScene a, String h, String i, String t, Map<String, Boolean> p)
	{
		this.card.setQuestion(q);
		this.card.setAnswer(a);
		this.card.setHint(h);
		this.card.setAdditionalInfo(i);
		this.card.setTitle(t);
		this.card.setOptions(p);
		setPreviousCard(this.card);
	}

	public void revertCard()
	{
		this.card.setQuestion(previous.getQuestion().getScene());
		this.card.setAnswer(previous.getAnswer().getScene());
		this.card.setHint(previous.getHint().getText());
		this.card.setAdditionalInfo(previous.getAdditionalInfo().getText());
		this.card.setTitle(previous.getTitle());
		this.card.setOptions(previous.getOptions());
	}
}
