package edu.jhu.cs.oose.elevel.testTaking;

import java.util.Random;

public abstract class ProbabilisticTestAlgorithm extends TestAlgorithm
{
	protected float[] probabilities;
	protected boolean[] cardSelected;
	protected Random rand;

	public ProbabilisticTestAlgorithm(int cardCount, boolean repeat, float[] probabilities)
	{
		super(cardCount, repeat);

		rand = new Random(System.currentTimeMillis());

		this.probabilities = probabilities;
		if(this.probabilities == null)
		{
			this.probabilities = new float[cardCount];
			for(int i = 0; i < this.probabilities.length; i++)
				this.probabilities[i] = 1.0F / cardCount;
		}
		normalizeProbabilities();

		cardSelected = new boolean[this.probabilities.length];
		for(int i = 0; i < cardSelected.length; i++)
			cardSelected[i] = false;
	}

	public void changeProbability(int index, float newProb)
	{
		probabilities[index] = newProb;
		normalizeProbabilities();
	}

	private void normalizeProbabilities()
	{
		float cummulativeSum = 0;

		for(int i = 0; i < probabilities.length; i++)
			cummulativeSum += probabilities[i];

		for(int i = 0; i < probabilities.length; i++)
			probabilities[i] /= cummulativeSum;
	}

	public boolean allCardsVisited()
	{
		for(int i = 0; i < cardSelected.length; i++)
			if(!cardSelected[i])
				return false;

		return true;
	}
}
