/**
 * TestAlgorithm.java
 * Group 7, Object Oriented Software Engineering
 * November 15, 2009
 */
package edu.jhu.cs.oose.elevel.testTaking;

/**
 * Provides an interface for a test's algorithm. All algorithms are required to provide an implementation
 * for nextCard().
 * 
 * @author Greg Anderson
 */
public abstract class TestAlgorithm
{
	/** The index of the current card */
	protected int currentCardIndex;

	/** The total number of cards in the deck */
	protected int cardCount = 0;

	/** A flag specifying whether the algorithm may repeat cards or not */
	protected boolean repeat = false;

	/**
	 * Instantiates a default TestAlgorithm
	 * 
	 * @param cardCount
	 *        an <code>int</code> representing the number of cards in the <code>Deck</code>
	 * @param repeat
	 *        a <code>boolean</code> representing whether or not cards may be repeated
	 */
	public TestAlgorithm(int cardCount, boolean repeat)
	{
		this.currentCardIndex = -1;
		this.cardCount = cardCount;
		this.repeat = repeat;
	}

	/**
	 * Returns the index of the next <code>Card</code> in the
	 * <code>Deck</card>. Implementation must be provided by subclasses.
	 * 
	 * @return an <code>int</code> representing the index of the next <code>Card</code>
	 */
	public abstract int nextCard();

	public void cardAnswered(boolean correct)
	{
		// do nothing.
		return;
	}
}
