/**
 * TestWindowGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel.testTaking;

import com.trolltech.qt.core.QRect;
import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QKeyEvent;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QToolTip;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.gui.QPalette.ColorRole;
import edu.jhu.cs.oose.elevel.AdvancedEditCard.DrawingGraphicsView;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.CardSection;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.dataModel.TestStat;
import edu.jhu.cs.oose.elevel.utilities.ClearingTextBox;
import edu.jhu.cs.oose.elevel.utilities.Face;

/**
 * GUI responsible for the view and controller for the underlying {@link TestWindow} model
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class TestWindowGUI extends QDialog
{
	/** The underlying {@link TestWindow} model */
	private Test test;

	/** The QLayout for this component */
	private QGridLayout layout;

	/** The Widget to display */
	private QWidget display;

	/** The area of the <code>card</code> to display */
	private QWidget cardDisplay;

	/** QAction to exit the window */
	private QAction quit;

	/** QPushButtons for inputs */
	private QPushButton hint;
	private QPushButton submit;
	private QPushButton next;
	private QPushButton correct;
	private QPushButton incorrect;
	private QPushButton done;

	/** The input area for the answer */
	private ClearingTextBox answerInput;

	/** The associated label for answer input */
	private QLabel answerLabel;

	/** Displays the percent right */
	private QLabel percentage;

	/** The current face displayed */
	private Face currentFace;

	private boolean correctAnswer;

	/**
	 * Initializes a new <code>TestWindowGUI</code> with the specified parameters
	 * 
	 * @param test
	 *        A {@link TestWindowGUI} representing the underlying model
	 * @param parent
	 *        A {@link QWidget} representing the parent window
	 * @return
	 */
	public TestWindowGUI(Test test, QWidget parent)
	{
		super(parent);
		this.setModal(true);
		setTestWindow(test);

		this.setBackgroundRole(ColorRole.Dark);
		this.setAutoFillBackground(true);
		// this.setGeometry(parent.geometry());
		System.out.println("Testing card: " + this.test.currentCard());

		this.layout = new QGridLayout();
		this.setLayout(this.layout);
		this.display = new QWidget();
		this.layout.addWidget(this.display);
		initalizeCard();

		this.addAction(this.quit);
		this.repaint();
	}

	/**
	 * Save the current deckStat to the {@link Deck} in the model for the test
	 * 
	 * @see com.trolltech.qt.gui.QDialog#closeEvent(com.trolltech.qt.gui.QCloseEvent)
	 */
	public void closeEvent(QCloseEvent event)
	{

	}

	private void finishAndDisplayStatistics()
	{
		TestStat d = this.test.finishTest();
		QMessageBox.information(this, tr("Test Statistics"), d.toString());
	}

	public void keyPressEvent(QKeyEvent event)
	{
		if(event.key() == Qt.Key.Key_Escape.value())
		{
			finishAndDisplayStatistics();
			this.close();
		}
	}

	/**
	 * Returns the {@link TestWindow} model this GUI is associated with
	 * 
	 * @return the {@link TestWindow} model this GUI is associated with
	 */
	public Test getTestWindow()
	{
		return this.test;
	}

	/**
	 * Initialize the parameters for the {@link Card}
	 */
	private void initalizeCard()
	{
		String s = this.test.getPreference("TEST_FRONT_TO_BACK");
		if(s == null || s.equals("true"))
			setFace(Face.FRONT);
		else
			setFace(Face.BACK);
		setupActions();
		setupButtons();
		setupFront();
	}

	/**
	 * Get the next <code>Card</code> or display the end of test prompt
	 */
	@SuppressWarnings("unused")
	private void nextCard()
	{
		if(!this.test.nextCard())
		{
			QMessageBox.StandardButton msg = QMessageBox.warning(this, tr("Test"), "You have completed the test!\n" + "Retake the test?", new QMessageBox.StandardButtons(QMessageBox.StandardButton.Yes, QMessageBox.StandardButton.No));

			if(msg == QMessageBox.StandardButton.Yes)
			{
				finishAndDisplayStatistics();
				this.test.initializeTest();
			}
			else if(msg == QMessageBox.StandardButton.No)
			{
				finishAndDisplayStatistics();
				this.close();
				return;
			}

		}

		nextFace();
		setupFront();
	}

	/**
	 * Determine the next face to show
	 */
	private void nextFace()
	{
		if(this.currentFace.equals(Face.FRONT))
			setFace(Face.BACK);
		else
			setFace(Face.FRONT);
	}

	/**
	 * Set the FACE information to the <code>Card</code> display area
	 * 
	 * @param f
	 *        the face of the card to be set
	 */
	private void setFace(Face f)
	{
		Card card = this.test.currentCard();
		if(card == null)
			return;

		// TODO switch this from a QTextEdit to a generic that is set by some other object (probably a toolbar)
		CardSection c;
		if(f.equals(Face.FRONT))
		{
			c = card.getQuestion();
			this.currentFace = Face.FRONT;
		}
		else
		{
			c = card.getAnswer();
			this.currentFace = Face.BACK;
		}

		if(c.hasText())
		{
			this.cardDisplay = new QTextEdit();
			((QTextEdit) this.cardDisplay).setText(c.getText());
			this.cardDisplay.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
			((QTextEdit) this.cardDisplay).setReadOnly(true);
		}
		else
		{
			this.cardDisplay = new QLabel();
			this.cardDisplay.setMinimumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
			this.cardDisplay.setMaximumSize(new QSize(CardSection.WIDTH, CardSection.HEIGHT));
			((QLabel) this.cardDisplay).setPixmap(new DrawingGraphicsView(c.getScene()).getImage());
		}
	}

	/**
	 * Associates this GUI with the specified {@link TestWindow} model
	 * 
	 * @param t
	 *        the {@link TestWindow} model associated with this GUI
	 */
	public void setTestWindow(Test t)
	{
		if(t != null)
			this.test = t;
	}

	/**
	 * Setup the actions
	 */
	private void setupActions()
	{
		this.quit = new QAction(tr("&Exit"), this);
		this.quit.setShortcut(tr("Ctrl+W"));
		this.quit.triggered.connect(this, "finishAndDisplayStatistics()");
		this.quit.triggered.connect(this, "close()");
	}

	/**
	 * Setup the back of the <code>Card</code>
	 */
	private void setupBack()
	{
		this.display.hide();
		QWidget back = new QWidget();
		QGridLayout backLayout = new QGridLayout();
		back.setLayout(backLayout);
		backLayout.addWidget(this.cardDisplay, 1, 0);// , AlignmentFlag.AlignCenter);

		if(this.test.currentCard().getOptions().get("Self-check Answer").booleanValue())
		{
			QHBoxLayout correctButtons = new QHBoxLayout();
			correctButtons.addWidget(this.correct);
			correctButtons.addWidget(this.incorrect);
			backLayout.addLayout(correctButtons, 0, 0);
		}
		else
		{
			this.answerInput.returnPressed.disconnect(this, "submit()");
			this.answerInput.returnPressed.connect(this, "nextCard()");

			QLabel correct = new QLabel("Incorrect!");
			if(correctAnswer)
			{
				correct.setText("Correct!");
			}
			backLayout.addWidget(correct, 0, 0, Qt.AlignmentFlag.AlignCenter);
			backLayout.addWidget(this.next, 0, 1);

		}

		if(this.test.getPreference("SHOW_SCORE").equalsIgnoreCase("true"))
		{
			String percent = this.test.getPercentSatus();
			percentage = new QLabel(percent + " Correct");
			backLayout.addWidget(percentage, 1, 1);
		}

		this.layout.removeWidget(this.display);
		this.display = back;
		this.layout.addWidget(this.display, 0, 0);
		this.display.show();
	}

	/**
	 * Setup the buttons/inputs for the GUI
	 */
	private void setupButtons()
	{
		this.next = new QPushButton(tr("&Next Card"));
		this.next.clicked.connect(this, "nextCard()");

		this.correct = new QPushButton(tr("&Right!"));
		this.correct.clicked.connect(this.test, "rightAnswer()");
		this.correct.clicked.connect(this, "nextCard()");
		this.correct.setObjectName("correct");
		this.correct.setStyleSheet("#correct { background-color: green}");

		this.incorrect = new QPushButton(tr("&Wrong"));
		this.incorrect.clicked.connect(this.test, "wrongAnswer()");
		this.incorrect.clicked.connect(this, "nextCard()");
		this.incorrect.setObjectName("incorrect");
		this.incorrect.setStyleSheet("#incorrect { background-color: red}");

		this.answerInput = new ClearingTextBox("Enter answer here");
		this.done = new QPushButton(tr("&Done"));
		this.done.clicked.connect(this, "finishAndDisplayStatistics()");
		this.done.clicked.connect(this, "close()");

		this.answerInput = new ClearingTextBox("Enter answer here");

		this.answerLabel = new QLabel("Answer:");
		this.answerLabel.setBackgroundRole(ColorRole.Link);

		// TODO decide if maybe we just want to use the ? button in the bar, rather than have a hint button
		// TODO make an icon for the hint
		this.hint = new QPushButton(tr("Hint"));
		this.hint.clicked.connect(this, "showHint()");

		this.submit = new QPushButton(tr("&Submit"));
		this.submit.clicked.connect(this, "submit()");
	}

	/**
	 * Setup the front of the <code>Card</code>
	 */
	private void setupFront()
	{
		this.display.hide();
		QWidget front = new QWidget();
		QGridLayout frontLayout = new QGridLayout();
		front.setLayout(frontLayout);

		frontLayout.addWidget(this.cardDisplay, 0, 0);// , AlignmentFlag.AlignCenter);
		frontLayout.addWidget(this.submit, 1, 1, Qt.AlignmentFlag.AlignCenter);

		QVBoxLayout buttons = new QVBoxLayout();
		frontLayout.addLayout(buttons, 0, 1, Qt.AlignmentFlag.AlignTop);
		if(this.test.currentCard().getOptions().get("Display Hints").booleanValue() && this.test.getPreference("ALLOW_HINTS").equalsIgnoreCase("true"))
		{
			buttons.addWidget(this.hint);
		}
		buttons.addWidget(this.done);

		frontLayout.setColumnMinimumWidth(6, 0);
		frontLayout.setRowMinimumHeight(6, 0);

		// System.out.println("Self-check Answers: " + this.card.options.get("Self-check Answers").booleanValue());
		if(!this.test.currentCard().getOptions().get("Self-check Answer").booleanValue())
		{
			QHBoxLayout answerLayout = new QHBoxLayout();
			answerLayout.addWidget(this.answerLabel);
			answerLayout.addWidget(this.answerInput);
			frontLayout.addLayout(answerLayout, 1, 0, Qt.AlignmentFlag.AlignCenter);
			this.answerInput.returnPressed.disconnect(this, "nextCard()");
			this.answerInput.returnPressed.connect(this, "submit()");
		}

		this.layout.removeWidget(this.display);
		this.display = front;
		this.layout.addWidget(this.display, 0, 0);
		this.display.show();
	}

	/**
	 * Shows the hint in a tooltip if clicked
	 */
	@SuppressWarnings("unused")
	private void showHint()
	{
		Card c = this.test.currentCard();
		CardSection h = c.getHint();
		if(h.hasText())
			QToolTip.showText(this.geometry().center(), h.getText(), this.hint, new QRect());
	}

	/**
	 * @see com.trolltech.qt.gui.QDialog#sizeHint()
	 */
	public QSize sizeHint()
	{
		return new QSize(200, 200);
	}

	/**
	 * Submits an answer and reveals the other face
	 */
	@SuppressWarnings("unused")
	private void submit()
	{
		if(!this.test.currentCard().getOptions().get("Self-check Answer").booleanValue())
		{
			if(this.test.checkAnswer(this.answerInput.text()))
			{
				correctAnswer = true;
			}
			else
			{
				correctAnswer = false;
			}
		}

		nextFace();
		setupBack();
	}
}
