/**
 * AnswerTextBox.java
 * Group 7, Object Oriented Software Engineering
 * November 18, 2009
 */
package edu.jhu.cs.oose.elevel.utilities;

import com.trolltech.qt.gui.QHideEvent;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QMouseEvent;

/**
 * Implements a QLineEdit capable of clearing its original text when selected
 * 
 * @author Jesse Yates
 */
public class ClearingTextBox extends QLineEdit
{
	/** Flag indicating whether the text is the original text */
	private boolean original;

	/** The default contents of the text bar */
	private String originalText;

	public ClearingTextBox(String string)
	{
		super(string);
		this.originalText = string;
		this.original = true;
	}

	/**
	 * Clears the original text when the text box is clicked
	 * 
	 * @see com.trolltech.qt.gui.QLineEdit#mousePressEvent(com.trolltech.qt.gui.QMouseEvent)
	 */
	@Override
	protected void mousePressEvent(QMouseEvent arg)
	{
		if(original)
		{
			this.clear();
			this.original = false;
		}
		super.mousePressEvent(arg);
	}

	/**
	 * Resets the answer box to the original text when hidden
	 * 
	 * @see com.trolltech.qt.gui.QLineEdit#mousePressEvent(com.trolltech.qt.gui.QMouseEvent)
	 */
	@Override
	protected void hideEvent(QHideEvent arg)
	{
		this.original = true;
		setText(this.originalText);
		super.hideEvent(arg);
	}
}
