/**
 * PreferenceButtonGroup.java
 * Group 7, Object Oriented Software Engineering
 * November 14, 2009
 */

package edu.jhu.cs.oose.elevel.utilities;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.gui.QAbstractButton;
import com.trolltech.qt.gui.QButtonGroup;

/**
 * Class to ensure that the name of the preference is included with its new state
 * 
 * @author H. Parker Shelton
 */
public class PreferenceButtonGroup extends QButtonGroup
{
	/** The preferences this QButtonGroup is associated with */
	private String preference;

	/** The Signal to connect to */
	public final QSignalEmitter.Signal2<String, String> stateChange = new QSignalEmitter.Signal2<String, String>();

	/**
	 * Constructor that takes the preference the button group is associated with
	 * 
	 * @param pref
	 *        A <code>String</code> representing the preference associated with this button group
	 */
	public PreferenceButtonGroup(String pref)
	{
		this.preference = pref;
		super.setExclusive(true);
		super.buttonClicked.connect(this, "wasClicked(QAbstractButton)");
	}

	/**
	 * Emits a signal that updates the specified preference with the name of the selected button
	 */
	public void wasClicked(QAbstractButton s)
	{
		stateChange.emit(this.preference, s.text().toUpperCase());
	}
}
