/**
 * PreferenceCheckBox.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */

package edu.jhu.cs.oose.elevel.utilities;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.gui.QCheckBox;

/**
 * Class to ensure that the name of the preference is included with its new state
 * 
 * @author Jesse Yates
 */
public class PreferenceCheckBox extends QCheckBox
{
	/** The preferences this checkbox is associated with */
	private String preference;

	/** The Signal to connect to */
	public final QSignalEmitter.Signal2<String, Boolean> stateChange = new QSignalEmitter.Signal2<String, Boolean>();

	/**
	 * Constructor that takes the name of the check box and the preference the checkbox is associated with
	 * 
	 * @param text
	 *        A <code>String</code> representing the text to display next to the checkbox
	 * @param pref
	 *        A <code>String</code> representing the preference associated with this checkbox
	 */
	public PreferenceCheckBox(String text, String pref)
	{
		super(text);
		this.preference = pref;
		clicked.connect(this, "wasClicked()");
	}

	/**
	 * Emits a signal that updates the specified preference with the new value of the checkbox
	 */
	public void wasClicked()
	{
		stateChange.emit(this.preference, new Boolean(this.isChecked()));
	}
}
